@@ -36,7 +36,7 @@ struct Varmatch
 
     virtual bool operator()(const wreport::Var&) const;
 
-    static std::auto_ptr<Varmatch> parse(const std::string& filter);
+    static std::unique_ptr<Varmatch> parse(const std::string& filter);
 };
 
 }
