@@ -1,28 +1,8 @@
-/*
- * dballe/db - Archive for point-based meteorological data
- *
- * Copyright (C) 2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
-#ifndef DBA_DB_H
-#define DBA_DB_H
+#ifndef DBALLE_DB_H
+#define DBALLE_DB_H
 
 #include <dballe/core/defs.h>
+#include <dballe/db/defs.h>
 #include <wreport/varinfo.h>
 #include <wreport/var.h>
 #include <vector>
@@ -36,47 +16,28 @@
  * Functions used to connect to DB-All.e and insert, query and delete data.
  */
 
-/**
- * Flags controlling message import
- * @{
- */
-/* Import the attributes. */
-#define DBA_IMPORT_ATTRS		1
-/* Attempt to merge pseudoana extra information into the existing ones. */
-#define DBA_IMPORT_FULL_PSEUDOANA	2
-/* Message data will overwrite existing values; otherwise, trying to insert
- * existing data will cause an error. */
-#define DBA_IMPORT_OVERWRITE		8
-/// @}
-
 namespace dballe {
 struct Record;
-struct Msg;
-struct Msgs;
-struct MsgConsumer;
+struct Query;
+struct Values;
+struct StationValues;
+struct DataValues;
+struct Message;
+struct Messages;
 struct DB;
 
 namespace db {
 struct Connection;
-
-/**
- * Supported formats
- */
-typedef enum {
-    V5 = 0,
-    V6 = 1,
-    MEM = 2,
-    MESSAGES = 3,
-} Format;
+struct ODBCConnection;
 
 /**
  * Simple typedef to make typing easier
  */
 typedef std::vector<wreport::Varcode> AttrList;
 
-class Cursor
+/// Common interface for all kinds of cursors
+struct Cursor
 {
-public:
     virtual ~Cursor();
 
     /// Get the database that created this cursor
@@ -123,17 +84,24 @@ public:
     virtual const char* get_ident(const char* def=0) const = 0;
 
     /// Get the report name
-    virtual const char* get_rep_memo(const char* def=0) const = 0;
+    virtual const char* get_rep_memo() const = 0;
 
-    /// Get the level
-    virtual Level get_level() const = 0;
+    /**
+     * Iterate the cursor until the end, returning the number of items.
+     *
+     * If dump is a FILE pointer, also dump the cursor values to it
+     */
+    virtual unsigned test_iterate(FILE* dump=0);
+};
 
-    /// Get the level
-    virtual Trange get_trange() const = 0;
-
-    /// Get the datetime
-    virtual void get_datetime(int (&dt)[6]) const = 0;
+/// Cursor iterating over stations
+struct CursorStation : public Cursor
+{
+};
 
+/// Common interface for cursors iterating over station or data values
+struct CursorValue : public Cursor
+{
     /// Get the variable code
     virtual wreport::Varcode get_varcode() const = 0;
 
@@ -145,19 +113,59 @@ public:
      * variable for attribute access
      */
     virtual int attr_reference_id() const = 0;
+};
 
+/// Cursor iterating over station data values
+struct CursorStationData : public CursorValue
+{
+};
+
+/// Cursor iterating over data values
+struct CursorData : public CursorValue
+{
+    /// Get the level
+    virtual Level get_level() const = 0;
+
+    /// Get the time range
+    virtual Trange get_trange() const = 0;
+
+    /// Get the datetime
+    virtual Datetime get_datetime() const = 0;
+};
+
+/// Cursor iterating over summary entries
+struct CursorSummary : public Cursor
+{
+    /// Get the level
+    virtual Level get_level() const = 0;
+
+    /// Get the time range
+    virtual Trange get_trange() const = 0;
+
+    /// Get the variable code
+    virtual wreport::Varcode get_varcode() const = 0;
+
+    /// Get the datetime range
+    virtual DatetimeRange get_datetimerange() const = 0;
+
+    /// Get the count of elements
+    virtual size_t get_count() const = 0;
+};
+
+
+#if 0
     /**
      * Query attributes for the current variable
      */
-    virtual unsigned query_attrs(const AttrList& qcs, Record& attrs) = 0;
+    virtual void query_attrs(std::function<void(std::unique_ptr<wreport::Var>&&)> dest) = 0;
 
     /**
      * Insert/overwrite new attributes for the current variable
      *
      * @param attrs
-     *   The record with the attributes to be added
+     *   The attributes to be added
      */
-    virtual void attr_insert(const Record& attrs) = 0;
+    virtual void attr_insert(const Values& attrs) = 0;
 
     /**
      * Delete attributes for the current variable
@@ -167,14 +175,7 @@ public:
      *   associated to id_data will be deleted.
      */
     virtual void attr_remove(const AttrList& qcs) = 0;
-
-    /**
-     * Iterate the cursor until the end, returning the number of items.
-     *
-     * If dump is a FILE pointer, also dump the cursor values to it
-     */
-    virtual unsigned test_iterate(FILE* dump=0);
-};
+#endif
 
 }
 
@@ -199,7 +200,7 @@ public:
      * @return
      *   The new DB object
      */
-    static std::auto_ptr<DB> connect(const char* dsn, const char* user, const char* password);
+    static std::unique_ptr<DB> connect(const char* dsn, const char* user, const char* password);
 
     /**
      * Create from a SQLite file pathname
@@ -207,7 +208,7 @@ public:
      * @param pathname
      *   The pathname to a SQLite file
      */
-    static std::auto_ptr<DB> connect_from_file(const char* pathname);
+    static std::unique_ptr<DB> connect_from_file(const char* pathname);
 
     /**
      * Create from an url-like specification, that can be:
@@ -219,12 +220,12 @@ public:
      * @param url
      *   The url-like connection descriptor
      */
-    static std::auto_ptr<DB> connect_from_url(const char* url);
+    static std::unique_ptr<DB> connect_from_url(const char* url);
 
     /**
      * Create an in-memory database
      */
-    static std::auto_ptr<DB> connect_memory(const std::string& arg = std::string());
+    static std::unique_ptr<DB> connect_memory(const std::string& arg = std::string());
 
     /**
      * Start a test session with DB-All.e
@@ -232,7 +233,12 @@ public:
      * Take information from the environment (@see dba_db_create_from_env) and
      * default to ./test.sqlite if nothing is specified.
      */
-    static std::auto_ptr<DB> connect_test();
+    static std::unique_ptr<DB> connect_test();
+
+    /**
+     * Create a database from an open Connection
+     */
+    static std::unique_ptr<DB> create(std::unique_ptr<db::Connection> conn);
 
     /**
      * Return TRUE if the string looks like a DB URL
@@ -266,7 +272,7 @@ public:
      *   If repinfo_file is NULL, then the default of /etc/dballe/repinfo.csv is
      *   used.
      */
-    virtual void reset(const char* repinfo_file = 0) = 0;
+    virtual void reset(const char* repinfo_file=0) = 0;
 
     /**
      * Update the repinfo table in the database, with the data found in the given
@@ -292,16 +298,15 @@ public:
      */
     virtual std::map<std::string, int> get_repinfo_priorities() = 0;
 
+public:
     /**
-     * Insert a record into the database
+     * Insert station values into the database
      *
-     * The reference IDs of all variables that were inserted will be stored in
-     * memory until the next insert operation. To insert attributes related to
-     * one of the variables just inserted, just call attr_insert() without the
-     * reference_id parameter.
+     * The IDs of the station andl all variables that were inserted will be
+     * stored in vals.
      *
-     * @param rec
-     *   The record to insert.
+     * @param vals
+     *   The values to insert.
      * @param can_replace
      *   If true, then existing data can be rewritten, else data can only be added.
      * @param station_can_add
@@ -310,12 +315,25 @@ public:
      *   data for a station that does not yet exists in the database, it will
      *   be created.
      */
-    virtual void insert(const Record& rec, bool can_replace, bool station_can_add) = 0;
+    virtual void insert_station_data(StationValues& vals, bool can_replace, bool station_can_add) = 0;
 
     /**
-     * Return the station id for the last data that was inserted.
+     * Insert data values into the database
+     *
+     * The IDs of the station andl all variables that were inserted will be
+     * stored in vals.
+     *
+     * @param vals
+     *   The values to insert.
+     * @param can_replace
+     *   If true, then existing data can be rewritten, else data can only be added.
+     * @param station_can_add
+     *   If false, it will not create a missing station record, and only data
+     *   for existing stations can be added. If true, then if we are inserting
+     *   data for a station that does not yet exists in the database, it will
+     *   be created.
      */
-    virtual int last_station_id() const = 0;
+    virtual void insert_data(DataValues& vals, bool can_replace, bool station_can_add) = 0;
 
     /**
      * Remove data from the database
@@ -324,7 +342,16 @@ public:
      *   The record with the query data (see technical specifications, par. 1.6.4
      *   "parameter output/input") to select the items to be deleted
      */
-    virtual void remove(const Record& rec) = 0;
+    virtual void remove_station_data(const Query& query) = 0;
+
+    /**
+     * Remove data from the database
+     *
+     * @param rec
+     *   The record with the query data (see technical specifications, par. 1.6.4
+     *   "parameter output/input") to select the items to be deleted
+     */
+    virtual void remove(const Query& rec) = 0;
 
     /**
      * Remove all data from the database.
@@ -358,7 +385,21 @@ public:
      * @return
      *   The cursor to use to iterate over the results
      */
-    virtual std::auto_ptr<db::Cursor> query_stations(const Record& query) = 0;
+    virtual std::unique_ptr<db::CursorStation> query_stations(const Query& query) = 0;
+
+    /**
+     * Query the station variables in the database.
+     *
+     * When multiple values per variable are present, the results will be presented
+     * in increasing order of priority.
+     *
+     * @param query
+     *   The record with the query data (see technical specifications, par. 1.6.4
+     *   "parameter output/input")
+     * @return
+     *   The cursor to use to iterate over the results
+     */
+    virtual std::unique_ptr<db::CursorStationData> query_station_data(const Query& query) = 0;
 
     /**
      * Query the database.
@@ -372,7 +413,7 @@ public:
      * @return
      *   The cursor to use to iterate over the results
      */
-    virtual std::auto_ptr<db::Cursor> query_data(const Record& rec) = 0;
+    virtual std::unique_ptr<db::CursorData> query_data(const Query& query) = 0;
 
     /**
      * Query a summary of what the result would be for a query.
@@ -386,72 +427,91 @@ public:
      *   provided, so it only gives all the available combinations of data
      *   contexts.
      */
-    virtual std::auto_ptr<db::Cursor> query_summary(const Record& rec) = 0;
+    virtual std::unique_ptr<db::CursorSummary> query_summary(const Query& query) = 0;
 
     /**
-     * Query attributes
+     * Query attributes on a station value
      *
-     * @param reference_id
-     *   The id (returned by Cursor::attr_reference_id()) used to refer to the variable we query
-     * @param id_var
-     *   The varcode of the variable related to the attributes to retrieve.  See @ref vartable.h
-     * @param qcs
-     *   The WMO codes of the QC values requested.  If it is empty, then all values
-     *   are returned.
-     * @param attrs
-     *   The Record that will hold the resulting attributes
-     * @return
-     *   Number of attributes returned in attrs
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
+     * @param dest
+     *   The function that will be called on each resulting attribute
      */
-    virtual unsigned query_attrs(int reference_id, wreport::Varcode id_var, const db::AttrList& qcs, Record& attrs) = 0;
+    virtual void attr_query_station(int data_id, std::function<void(std::unique_ptr<wreport::Var>)>&& dest) = 0;
 
     /**
-     * Insert new attributes into the database, reusing the reference IDs stored by the last insert.
+     * Query attributes on a data value
      *
-     * @param id_var
-     *   The varcode of the variable related to the attributes to add.  See @ref vartable.h
-     * @param attrs
-     *   The record with the attributes to be added
-     * @param can_replace
-     *   If true, then existing data can be rewritten, else data can only be added.
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
+     * @param dest
+     *   The function that will be called on each resulting attribute
      */
-    virtual void attr_insert(wreport::Varcode id_var, const Record& attrs) = 0;
+    virtual void attr_query_data(int data_id, std::function<void(std::unique_ptr<wreport::Var>)>&& dest) = 0;
 
     /**
-     * Insert new attributes into the database.
+     * Insert new attributes on a station value
      *
-     * @param reference_id
-     *   The id (returned by Cursor::attr_reference_id()) used to refer to the variable we query
-     * @param id_var
-     *   The varcode of the variable related to the attributes to add.  See @ref vartable.h
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
      * @param attrs
-     *   The record with the attributes to be added
-     * @param can_replace
-     *   If true, then existing data can be rewritten, else data can only be added.
+     *   The attributes to be added
      */
-    virtual void attr_insert(int reference_id, wreport::Varcode id_var, const Record& attrs) = 0;
+    virtual void attr_insert_station(int data_id, const Values& attrs) = 0;
 
     /**
-     * Delete QC data for the variable `var' in record `rec' (coming from a previous
-     * dba_query)
+     * Insert new attributes on a data value
      *
-     * @param reference_id
-     *   The id (returned by Cursor::attr_reference_id()) used to refer to the variable we query
-     * @param id_var
-     *   The varcode of the variable related to the attributes to remove.  See @ref vartable.h
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
+     * @param attrs
+     *   The attributes to be added
+     */
+    virtual void attr_insert_data(int data_id, const Values& attrs) = 0;
+
+    /**
+     * Delete attributes from a station value
+     *
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
      * @param qcs
      *   Array of WMO codes of the attributes to delete.  If empty, all attributes
-     *   associated to id_data will be deleted.
+     *   associated to the value will be deleted.
      */
-    virtual void attr_remove(int reference_id, wreport::Varcode id_var, const db::AttrList& qcs) = 0;
+    virtual void attr_remove_station(int data_id, const db::AttrList& qcs) = 0;
 
     /**
-     * Import a Msg message into the DB-All.e database
+     * Delete attributes from a data value
+     *
+     * @param data_id
+     *   The id (returned by Cursor::attr_reference_id()) used to refer to the value
+     * @param qcs
+     *   Array of WMO codes of the attributes to delete.  If empty, all attributes
+     *   associated to the value will be deleted.
+     */
+    virtual void attr_remove_data(int data_id, const db::AttrList& qcs) = 0;
+
+    /**
+     * Check if this varcode and data_id correspond to a station variable. This
+     * is used only to implement attr_* functions to be used when this
+     * information is not known. A database for which this information is not
+     * relevant (for example, in v6 databases there is a single ID space for
+     * station and data variable) can return any arbitrary value, since later
+     * they will be ignored.
+     *
+     * This should disappear once client APIs start making a distinction
+     * between attributes in station variables and attributes on data
+     * variables.
+     */
+    virtual bool is_station_variable(int data_id, wreport::Varcode varcode) = 0;
+
+    /**
+     * Import a Message into the DB-All.e database
      *
      * @param db
      *   The DB-All.e database to write the data into
      * @param msg
-     *   The Msg containing the data to import
+     *   The Message containing the data to import
      * @param repmemo
      *   Report mnemonic to which imported data belong.  If NULL is passed, then it
      *   will be chosen automatically based on the message type.
@@ -459,15 +519,15 @@ public:
      *   Customise different aspects of the import process.  It is a bitmask of the
      *   various DBA_IMPORT_* macros.
      */
-    virtual void import_msg(const Msg& msg, const char* repmemo, int flags) = 0;
+    virtual void import_msg(const Message& msg, const char* repmemo, int flags) = 0;
 
     /**
-     * Import Msgs messages into the DB-All.e database
+     * Import Messages into the DB-All.e database
      *
      * @param db
      *   The DB-All.e database to write the data into
      * @param msgs
-     *   The Msgs containing the data to import
+     *   The Messages containing the data to import
      * @param repmemo
      *   Report mnemonic to which imported data belong.  If NULL is passed, then it
      *   will be chosen automatically based on the message type.
@@ -475,18 +535,21 @@ public:
      *   Customise different aspects of the import process.  It is a bitmask of the
      *   various DBA_IMPORT_* macros.
      */
-    virtual void import_msgs(const Msgs& msgs, const char* repmemo, int flags);
+    virtual void import_msgs(const Messages& msgs, const char* repmemo, int flags);
 
     /**
-     * Perform the query in `query', and return the results as a NULL-terminated
-     * array of dba_msg.
+     * Perform the query in `query', and send the results to dest.
+     *
+     * Return false from dest to interrupt the query.
      *
      * @param query
      *   The query to perform
-     * @param cons
-     *   The MsgsConsumer that will handle the resulting messages
+     * @param dest
+     *   The function that will handle the resulting messages
+     * @returns true if the query reached its end, false if it got interrupted
+     *   because dest returned false.
      */
-    virtual void export_msgs(const Record& query, MsgConsumer& cons) = 0;
+    virtual bool export_msgs(const Query& query, std::function<bool(std::unique_ptr<Message>&&)> dest) = 0;
 
     /**
      * Dump the entire contents of the database to an output stream
@@ -495,9 +558,6 @@ public:
 
     /// Return the default repinfo file pathname
     static const char* default_repinfo_file();
-
-protected:
-    static std::auto_ptr<DB> instantiate_db(std::auto_ptr<db::Connection>& conn);
 };
 
 }
