@@ -1,7 +1,7 @@
 /*
  * dballe/aof_codec - AOF import
  *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 #include "aof_codec.h"
 #include "aof_importers/common.h"
 #include "msg.h"
-#include <dballe/core/file.h>
-#include <dballe/msg/msgs.h>
+#include "dballe/file.h"
+#include "dballe/message.h"
 #include <wreport/conv.h>
 
 #include <cstdlib>
@@ -43,19 +43,19 @@ AOFImporter::AOFImporter(const Options& opts)
     : Importer(opts) {}
 AOFImporter::~AOFImporter() {}
 
-void AOFImporter::from_rawmsg(const Rawmsg& msg, Msgs& msgs) const
+bool AOFImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const
 {
     /* char id[10]; */
     TRACE("aof_message_decode\n");
 
     /* Access the raw data in a more comfortable form */
-    const uint32_t* obs = (const uint32_t*)msg.data();
-    int obs_len = msg.size() / sizeof(uint32_t);
+    const uint32_t* obs = (const uint32_t*)msg.data.data();
+    int obs_len = msg.data.size() / sizeof(uint32_t);
 
     TRACE("05 grid box number: %d\n", OBS(5));
     TRACE("obs type: %d, %d\n", OBS(6), OBS(7));
 
-    auto_ptr<Msg> out(new Msg);
+    unique_ptr<Msg> out(new Msg);
 
 #if 0
     /* 13 Station ID (1:4) */
@@ -80,38 +80,38 @@ void AOFImporter::from_rawmsg(const Rawmsg& msg, Msgs& msgs) const
         case 6: read_pilot(obs, obs_len, *out); break;
         case 7: read_satem(obs, obs_len, *out); break;
         default:
-            error_parse::throwf(msg.file.c_str(), msg.offset,
+            error_parse::throwf(msg.pathname.c_str(), msg.offset,
                     "cannot handle AOF observation type %d subtype %d",
                     OBS(5), OBS(6));
     }
 
-    msgs.acquire(out);
+    return dest(unique_ptr<Message>(out.release()));
 }
 
-void AOFImporter::from_bulletin(const wreport::Bulletin&, Msgs&) const
+Messages AOFImporter::from_bulletin(const wreport::Bulletin&) const
 {
     throw error_unimplemented("AOF importer cannot import from wreport::Bulletin");
 }
 
-void AOFImporter::get_category(const Rawmsg& msg, int* category, int* subcategory)
+void AOFImporter::get_category(const BinaryMessage& msg, int* category, int* subcategory)
 {
     /* Access the raw data in a more comfortable form */
-    const uint32_t* obs = (const uint32_t*)msg.data();
-    int obs_len = msg.size() / sizeof(uint32_t);
+    const uint32_t* obs = (const uint32_t*)msg.data.data();
+    int obs_len = msg.data.size() / sizeof(uint32_t);
 
     if (obs_len < 7)
-        throw error_parse(msg.file.c_str(), msg.offset,
+        throw error_parse(msg.pathname.c_str(), msg.offset,
                 "the buffer is too short to contain an AOF message");
 
     *category = obs[5];
     *subcategory = obs[6];
 }
 
-void AOFImporter::dump(const Rawmsg& msg, FILE* out)
+void AOFImporter::dump(const BinaryMessage& msg, FILE* out)
 {
     /* Access the raw data in a more comfortable form */
-    const uint32_t* obs = (const uint32_t*)msg.data();
-    int obs_len = msg.size() / sizeof(uint32_t);
+    const uint32_t* obs = (const uint32_t*)msg.data.data();
+    int obs_len = msg.data.size() / sizeof(uint32_t);
 
     for (int i = 0; i < obs_len; i++)
         if (obs[i] == 0x7fffffff)
@@ -218,11 +218,18 @@ int AOFImporter::parse_lat_lon_datetime(const uint32_t* obs, Msg& msg)
     /* 12 Exact time of observation */
     parse_obs_datetime(obs, &datetime);
 
-    msg.set_year(datetime.tm_year + 1900, get_conf6((OBS(19) >> 12) & 0x3f));
-    msg.set_month(datetime.tm_mon + 1, get_conf6((OBS(19) >> 12) & 0x3f));
-    msg.set_day(datetime.tm_mday, get_conf6((OBS(19) >> 12) & 0x3f));
-    msg.set_hour(datetime.tm_hour, get_conf6((OBS(19) >> 18) & 0x3f));
-    msg.set_minute(datetime.tm_min, get_conf6((OBS(19) >> 18) & 0x3f));
+    msg.set_datetime(Datetime(
+        datetime.tm_year + 1900,
+        datetime.tm_mon + 1,
+        datetime.tm_mday,
+        datetime.tm_hour,
+        datetime.tm_min,
+        0));
+    msg.seti(WR_VAR(0, 4, 1), datetime.tm_year + 1900, get_conf6((OBS(19) >> 12) & 0x3f), Level(), Trange());
+    msg.seti(WR_VAR(0, 4, 2), datetime.tm_mon + 1, get_conf6((OBS(19) >> 12) & 0x3f), Level(), Trange());
+    msg.seti(WR_VAR(0, 4, 3), datetime.tm_mday, get_conf6((OBS(19) >> 12) & 0x3f), Level(), Trange());
+    msg.seti(WR_VAR(0, 4, 4), datetime.tm_hour, get_conf6((OBS(19) >> 18) & 0x3f), Level(), Trange());
+    msg.seti(WR_VAR(0, 4, 5), datetime.tm_min, get_conf6((OBS(19) >> 18) & 0x3f), Level(), Trange());
 
     return datetime.tm_hour;
 }
