@@ -1,24 +1,3 @@
-/*
- * dballe/aof_codec - AOF import
- *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
 #ifndef DBALLE_AOF_CODEC_H
 #define DBALLE_AOF_CODEC_H
 
@@ -89,22 +68,9 @@ public:
     AOFImporter(const Options& opts=Options());
     virtual ~AOFImporter();
 
-    /**
-     * Decode a message from its raw encoded representation
-     *
-     * @param rmsg
-     *   Encoded message
-     * @retval msgs
-     *   The resulting ::dba_msg
-     * @return
-     *   The error indicator for the function. See @ref error.h
-     */
-    virtual void from_rawmsg(const Rawmsg& msg, Msgs& msgs) const;
+    bool foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const override;
 
-    /**
-     * Import a decoded BUFR/CREX message
-     */
-    virtual void from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const;
+    Messages from_bulletin(const wreport::Bulletin& msg) const override;
 
     /**
      * Get category and subcategory of an AOF message
@@ -116,7 +82,7 @@ public:
      * @retval subcategory
      *   The AOF subcategory of the message
      */
-    static void get_category(const Rawmsg& msg, int* category, int* subcategory);
+    static void get_category(const BinaryMessage& msg, int* category, int* subcategory);
 
     /**
      * Print the contents of the AOF message
@@ -126,7 +92,7 @@ public:
      * @param out
      *   The stream to use to print the message
      */
-    static void dump(const Rawmsg& msg, FILE* out);
+    static void dump(const BinaryMessage& msg, FILE* out);
 };
 
 } // namespace msg
