@@ -1,32 +1,11 @@
-/*
- * msg/codec - General codec options
- *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
 #ifndef DBA_MSG_CODEC_H
 #define DBA_MSG_CODEC_H
 
-#include <dballe/core/rawmsg.h>
-#include <dballe/core/defs.h>
+#include <dballe/file.h>
+#include <dballe/message.h>
 #include <memory>
 #include <string>
-#include <stdio.h>
+#include <cstdio>
 
 /** @file
  * @ingroup msg
@@ -38,8 +17,8 @@ struct Bulletin;
 }
 
 namespace dballe {
-struct Rawmsg;
-struct Msgs;
+struct Messages;
+struct Message;
 
 namespace msg {
 
@@ -89,16 +68,30 @@ public:
      * @retval msgs
      *   The resulting ::dba_msg
      */
-    virtual void from_rawmsg(const Rawmsg& msg, Msgs& msgs) const = 0;
+    Messages from_binary(const BinaryMessage& msg) const;
+
+    /**
+     * Decode a message from its raw encoded representation, calling \a dest on
+     * each resulting Message.
+     *
+     * Return false from \a dest to stop decoding.
+     *
+     * @param rmsg
+     *   Encoded message.
+     * @retval dest
+     *   The function that consumes the decoded messages.
+     * @returns true if it got to the end of decoding, false if dest returned false.
+     */
+    virtual bool foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const = 0;
 
     /**
      * Import a decoded BUFR/CREX message
      */
-    virtual void from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const = 0;
+    virtual Messages from_bulletin(const wreport::Bulletin& msg) const = 0;
 
 
     /// Instantiate the right importer for the given type
-    static std::auto_ptr<Importer> create(Encoding type, const Options& opts=Options());
+    static std::unique_ptr<Importer> create(File::Encoding type, const Options& opts=Options());
 };
 
 /**
@@ -146,14 +139,14 @@ public:
      * @param msgs
      *   Message to encode
      * @retval rmsg
-     *   The resulting Rawmsg
+     *   The resulting BinaryMessage
      */
-    virtual void to_rawmsg(const Msgs& msgs, Rawmsg& msg) const = 0;
+    virtual std::string to_binary(const Messages& msgs) const = 0;
 
     /**
      * Export to a Bulletin
      */
-    virtual void to_bulletin(const Msgs& msgs, wreport::Bulletin& msg) const = 0;
+    virtual std::unique_ptr<wreport::Bulletin> to_bulletin(const Messages& msgs) const = 0;
 
     /**
      * Create a bulletin that works with this exporter.
@@ -161,15 +154,14 @@ public:
      * @returns the bulletin, or NULL of this is an exporter for a format not
      * covered by Bulletin
      */
-    virtual std::auto_ptr<wreport::Bulletin> make_bulletin() const;
+    virtual std::unique_ptr<wreport::Bulletin> make_bulletin() const;
 
 
     /// Instantiate the right importer for the given type
-    static std::auto_ptr<Exporter> create(Encoding type, const Options& opts=Options());
+    static std::unique_ptr<Exporter> create(File::Encoding type, const Options& opts=Options());
 };
 
-} // namespace msg
-} // namespace dballe
+}
+}
 
-/* vim:set ts=4 sw=4: */
 #endif
