@@ -82,7 +82,7 @@ Context& Context::operator=(const Context& src)
 
 bool Context::is_station() const
 {
-    return level == Level::ana() && trange == Trange::ana();
+    return level == Level() && trange == Trange();
 }
 
 int Context::compare(const Context& ctx) const
@@ -101,7 +101,7 @@ int Context::compare(const Level& lev, const Trange& tr) const
 
 void Context::set(const Var& var)
 {
-    set(auto_ptr<Var>(new Var(var)));
+    set(unique_ptr<Var>(new Var(var)));
 }
 
 void Context::seti(wreport::Varcode code, int val)
@@ -119,7 +119,7 @@ void Context::setc(wreport::Varcode code, const char* val)
     set(newvar(code, val));
 }
 
-void Context::set(auto_ptr<Var> var)
+void Context::set(unique_ptr<Var> var)
 {
     Varcode code = var->code();
     int idx = find_index(code);
@@ -181,7 +181,7 @@ Var* Context::edit(Varcode code)
 bool Context::remove(Varcode code)
 {
     int idx = find_index(code);
-    if (idx == -1) return NULL;
+    if (idx == -1) return false;
     Var* res = data[idx];
     data.erase(data.begin() + idx);
     delete res;
@@ -261,7 +261,7 @@ unsigned Context::diff(const Context& ctx) const
                 ++i1;
                 ++i2;
             } else if (cmp < 0) {
-                if (data[i1]->value() != NULL)
+                if (data[i1]->isset())
                 {
                     context_var_summary(*this, *data[i1], notes::log());
                     notes::log() << " exists only in the first message" << endl;
@@ -269,7 +269,7 @@ unsigned Context::diff(const Context& ctx) const
                 }
                 ++i1;
             } else {
-                if (ctx.data[i2]->value() != NULL)
+                if (ctx.data[i2]->isset())
                 {
                     context_var_summary(ctx, *ctx.data[i2], notes::log());
                     notes::log() << " exists only in the second message" << endl;
@@ -292,7 +292,7 @@ const Var* Context::find_vsig() const
     if (res == NULL) return NULL;
 
     // Ensure it is not undefined
-    if (res->value() == NULL) return NULL;
+    if (!res->isset()) return NULL;
 
     // Finally return it
     return res;
@@ -300,5 +300,3 @@ const Var* Context::find_vsig() const
 
 }
 }
-
-/* vim:set ts=4 sw=4: */
