@@ -22,7 +22,8 @@
 #include "msg.h"
 #include "context.h"
 #include "dballe/msg/vars.h"
-#include <dballe/core/csv.h>
+#include "dballe/core/var.h"
+#include "dballe/core/csv.h"
 #include <wreport/codetables.h>
 #include <wreport/notes.h>
 
@@ -38,6 +39,48 @@ using namespace std;
 
 namespace dballe {
 
+namespace msg {
+
+Messages messages_from_csv(CSVReader& in)
+{
+    Messages res;
+    string old_rep;
+    bool first = true;
+    while (true)
+    {
+        // Seek to beginning, skipping empty lines
+        if (!in.move_to_data())
+            return res;
+
+        if (in.cols.size() != 13)
+            error_consistency::throwf("cannot parse CSV line has %zd fields instead of 13", in.cols.size());
+        if (first)
+        {
+            // If we are the first run, initialse old_* markers with the contents of this line
+            old_rep = in.cols[2];
+            first = false;
+        } else if (old_rep != in.cols[2])
+            // If Report changes, we are done
+            break;
+
+        unique_ptr<Msg> msg(new Msg);
+        bool has_next = msg->from_csv(in);
+        res.append(std::move(msg));
+        if (!has_next)
+            break;
+    }
+    return res;
+}
+
+void messages_to_csv(const Messages& msgs, std::ostream& out)
+{
+    for (const auto& i: msgs)
+        Msg::downcast(i).to_csv(out);
+}
+
+}
+
+
 const char* msg_type_name(MsgType type)
 {
     switch (type)
@@ -71,7 +114,7 @@ Msg::~Msg()
 }
 
 Msg::Msg(const Msg& m)
-    : type(m.type)
+    : m_datetime(m.m_datetime), type(m.type)
 {
     // Reserve space for the new contexts
     data.reserve(m.data.size());
@@ -87,6 +130,7 @@ Msg& Msg::operator=(const Msg& m)
     // Manage a = a
     if (this == &m) return *this;
 
+    m_datetime = m.m_datetime;
     type = m.type;
 
     // Delete existing vars
@@ -105,6 +149,27 @@ Msg& Msg::operator=(const Msg& m)
     return *this;
 }
 
+const Msg& Msg::downcast(const Message& o)
+{
+    const Msg* ptr = dynamic_cast<const Msg*>(&o);
+    if (!ptr)
+        throw error_consistency("Message given is not a Msg");
+    return *ptr;
+}
+
+Msg& Msg::downcast(Message& o)
+{
+    Msg* ptr = dynamic_cast<Msg*>(&o);
+    if (!ptr)
+        throw error_consistency("Message given is not a Msg");
+    return *ptr;
+}
+
+std::unique_ptr<Message> Msg::clone() const
+{
+    return unique_ptr<Message>(new Msg(*this));
+}
+
 void Msg::clear()
 {
     type = MSG_GENERIC;
@@ -142,7 +207,7 @@ const msg::Context* Msg::find_context(const Level& lev, const Trange& tr) const
 
 const msg::Context* Msg::find_station_context() const
 {
-    return find_context(Level::ana(), Trange::ana());
+    return find_context(Level(), Trange());
 }
 
 msg::Context* Msg::edit_context(const Level& lev, const Trange& tr)
@@ -155,7 +220,7 @@ msg::Context* Msg::edit_context(const Level& lev, const Trange& tr)
 
 msg::Context& Msg::obtain_station_context()
 {
-    return obtain_context(Level::ana(), Trange::ana());
+    return obtain_context(Level(), Trange());
 }
 
 msg::Context& Msg::obtain_context(const Level& lev, const Trange& tr)
@@ -163,15 +228,15 @@ msg::Context& Msg::obtain_context(const Level& lev, const Trange& tr)
     int pos = find_index(lev, tr);
     if (pos == -1)
     {
-        auto_ptr<msg::Context> c(new msg::Context(lev, tr));
+        unique_ptr<msg::Context> c(new msg::Context(lev, tr));
         msg::Context* res = c.get();
-        add_context(c);
+        add_context(std::move(c));
         return *res;
     }
     return *data[pos];
 }
 
-void Msg::add_context(auto_ptr<msg::Context> ctx)
+void Msg::add_context(unique_ptr<msg::Context>&& ctx)
 {
     // Enlarge the data
     data.resize(data.size() + 1);
@@ -205,7 +270,7 @@ bool Msg::remove_context(const Level& lev, const Trange& tr)
     return true;
 }
 
-const Var* Msg::find(Varcode code, const Level& lev, const Trange& tr) const
+const Var* Msg::get(Varcode code, const Level& lev, const Trange& tr) const
 {
     const msg::Context* ctx = find_context(lev, tr);
     if (ctx == NULL) return NULL;
@@ -219,10 +284,20 @@ wreport::Var* Msg::edit(wreport::Varcode code, const Level& lev, const Trange& t
     return ctx->edit(code);
 }
 
+bool Msg::remove(wreport::Varcode code, const Level& lev, const Trange& tr)
+{
+    msg::Context* ctx = edit_context(lev, tr);
+    if (!ctx) return false;
+    if (!ctx->remove(code)) return false;
+    if (ctx->data.empty())
+        remove_context(lev, tr);
+    return true;
+}
+
 const Var* Msg::find_by_id(int id) const
 {
     const MsgVarShortcut& v = shortcutTable[id];
-    return find(v.code, Level(v.ltype1, v.l1, v.ltype2, v.l2), Trange(v.pind, v.p1, v.p2));
+    return get(v.code, Level(v.ltype1, v.l1, v.ltype2, v.l2), Trange(v.pind, v.p1, v.p2));
 }
 
 const msg::Context* Msg::find_context_by_id(int id) const
@@ -241,29 +316,18 @@ namespace {
 
 struct VarContext
 {
+    const Msg& msg;
     // Extract datetime, lat, lon
     const Var* lat;
     const Var* lon;
-    const Var* year;
-    const Var* month;
-    const Var* day;
-    const Var* hour;
-    const Var* minute;
-    const Var* second;
     const Var* memo;
     const char* rep_memo;
 
-    VarContext(const Msg& m)
+    VarContext(const Msg& m) : msg(m)
     {
         // Extract datetime, lat, lon
         lat = m.get_latitude_var();
         lon = m.get_longitude_var();
-        year = m.get_year_var();
-        month = m.get_month_var();
-        day = m.get_day_var();
-        hour = m.get_hour_var();
-        minute = m.get_minute_var();
-        second = m.get_second_var();
         memo = m.get_rep_memo_var();
         if (memo)
             rep_memo = memo->enqc();
@@ -288,22 +352,18 @@ struct VarContext
         // Report type
         out << rep_memo << ",";
 
-        if (c.level != Level::ana())
+        if (c.level != Level())
         {
             // Datetime
-            out << setfill('0') << setw(4) << (year ? year->enq(0) : 0) << "-";
-            out << setfill('0') << setw(2) << (month ? month->enq(0) : 0) << "-";
-            out << setfill('0') << setw(2) << (day ? day->enq(0) : 0) << " ";
-            out << setfill('0') << setw(2) << (hour ? hour->enq(0) : 0) << ":";
-            out << setfill('0') << setw(2) << (minute ? minute->enq(0) : 0) << ":";
-            out << setfill('0') << setw(2) << (second ? second->enq(0) : 0) << ",";
+            msg.get_datetime().to_stream_iso8601(out, ' ');
+            out << ',';
 
             // Level
-            c.level.format(out, "");
+            c.level.to_stream(out, "");
             out << ",";
 
             // Time range
-            c.trange.format(out, "");
+            c.trange.to_stream(out, "");
             out << ",";
         } else
             out << ",,,,,,,,";
@@ -327,7 +387,7 @@ void Msg::to_csv(std::ostream& out) const
 
             vc.print(out, c);
 
-            out << format_code(v.code()) << ","; // B code
+            out << varcode_format(v.code()) << ","; // B code
             csv_output_quoted_string(out, v.format(""));
             out << endl;
 
@@ -335,7 +395,7 @@ void Msg::to_csv(std::ostream& out) const
             for (const Var* a = v.next_attr(); a != NULL; a = a->next_attr())
             {
                 vc.print(out, c);
-                out << format_code(v.code()) << "." << format_code(a->code()) << ","; // B code
+                out << varcode_format(v.code()) << "." << varcode_format(a->code()) << ","; // B code
                 csv_output_quoted_string(out, a->format(""));
                 out << endl;
             }
@@ -348,6 +408,25 @@ void Msg::csv_header(std::ostream& out)
     out << "Longitude,Latitude,Report,Date,Level1,L1,Level2,L2,Time range,P1,P2,Varcode,Value" << endl;
 }
 
+namespace {
+// Convert a string to an integer value, returning MISSING_INT if the string is
+// empty or "-"
+int str_to_int(const char* str)
+{
+    if (str == NULL || str[0] == 0 || strcmp(str, "-") == 0)
+        return MISSING_INT;
+    else
+        return strtol(str, NULL, 10);
+}
+int str_to_int(const std::string& str)
+{
+    if (str.empty() || str == "-")
+        return MISSING_INT;
+    else
+        return stoi(str);
+}
+}
+
 bool Msg::from_csv(CSVReader& in)
 {
     // Seek to beginning, skipping empty lines
@@ -374,7 +453,7 @@ bool Msg::from_csv(CSVReader& in)
             set_longitude(strtod(old_lon.c_str(), NULL));
             set_rep_memo(old_rep.c_str());
             if (!old_date.empty())
-                set_date(old_date.c_str());
+                set_datetime(Datetime::from_iso8601(old_date.c_str()));
             type = type_from_repmemo(old_rep.c_str());
             first = false;
         } else if (old_lon != in.cols[0] || old_lat != in.cols[1] || old_rep != in.cols[2]) {
@@ -387,7 +466,7 @@ bool Msg::from_csv(CSVReader& in)
             {
                 // previous lines were station information, next line is data
                 old_date = in.cols[3];
-                set_date(old_date.c_str());
+                set_datetime(Datetime::from_iso8601(old_date.c_str()));
             }
             else if (in.cols[3].empty())
                 // previous lines were data, next line is station information
@@ -401,33 +480,32 @@ bool Msg::from_csv(CSVReader& in)
         // out << "Longitude,Latitude,Report,Date,Level1,L1,Level2,L2,Time range,P1,P2,Varcode,Value" << endl;
 
         // Acquire the data
-        Level lev(in.cols[4].c_str(), in.cols[5].c_str(), in.cols[6].c_str(), in.cols[7].c_str());
+        Level lev(str_to_int(in.cols[4]), str_to_int(in.cols[5]), str_to_int(in.cols[6]), str_to_int(in.cols[7]));
         if (in.cols[3].empty())
             // If we have station info, set level accordingly
-            lev = Level::ana();
-        Trange tr(in.cols[8].c_str(), in.cols[9].c_str(), in.cols[10].c_str());
+            lev = Level();
+        Trange tr(str_to_int(in.cols[8]), str_to_int(in.cols[9]), str_to_int(in.cols[10]));
 
         // Parse variable code
         if (in.cols[11].size() == 13)
         {
             // Bxxyyy.Bxxyyy: attribute
-            Varcode vcode = descriptor_code(in.cols[11].substr(0, 6).c_str());
+            Varcode vcode = varcode_parse(in.cols[11].substr(0, 6).c_str());
             // Find master variable
             wreport::Var* var = edit(vcode, lev, tr);
             if (var == NULL)
                 error_consistency::throwf("cannot find corresponding variable for attribute %s", in.cols[11].c_str());
 
-            Varcode acode = descriptor_code(in.cols[11].substr(7).c_str());
-            auto_ptr<Var> attr = newvar(acode);
-            Varinfo info = attr->info();
-            attr->set_from_formatted(in.cols[12].c_str());
-            var->seta(attr);
+            Varcode acode = varcode_parse(in.cols[11].substr(7).c_str());
+            auto attr = newvar(acode);
+            attr->setf(in.cols[12].c_str());
+            var->seta(move(attr));
         } else if (in.cols[11].size() == 6) {
             // Bxxyyy: variable
-            Varcode vcode = descriptor_code(in.cols[11].c_str());
-            auto_ptr<Var> var = newvar(vcode);
-            var->set_from_formatted(in.cols[12].c_str());
-            set(var, lev, tr);
+            Varcode vcode = varcode_parse(in.cols[11].c_str());
+            unique_ptr<Var> var = newvar(vcode);
+            var->setf(in.cols[12].c_str());
+            set(std::move(var), lev, tr);
         } else
             error_consistency::throwf("cannot parse variable code %s", in.cols[11].c_str());
 
@@ -496,8 +574,10 @@ static void context_summary(const msg::Context& c, ostream& out)
     out << "c(" << c.level << ", " << c.trange << ")";
 }
 
-unsigned Msg::diff(const Msg& msg) const
+unsigned Msg::diff(const Message& o) const
 {
+    const Msg& msg = downcast(o);
+
     unsigned diffs = 0;
     if (type != msg.type)
     {
@@ -564,47 +644,34 @@ void Msg::set(const Var& var, Varcode code, const Level& lev, const Trange& tr)
     set(var_copy_without_unset_attrs(var, code), lev, tr);
 }
 
-void Msg::set(std::auto_ptr<Var> var, const Level& lev, const Trange& tr)
+void Msg::set(std::unique_ptr<Var>&& var, const Level& lev, const Trange& tr)
 {
     msg::Context& ctx = obtain_context(lev, tr);
-    ctx.set(var);
+    ctx.set(std::move(var));
 }
 
 void Msg::seti(Varcode code, int val, int conf, const Level& lev, const Trange& tr)
 {
-    auto_ptr<Var> var(newvar(code, val));
+    unique_ptr<Var> var(newvar(code, val));
     if (conf != -1)
-        var->seta(auto_ptr<Var>(newvar(WR_VAR(0, 33, 7), conf)));
-    set(var, lev, tr);
+        var->seta(newvar(WR_VAR(0, 33, 7), conf));
+    set(std::move(var), lev, tr);
 }
 
 void Msg::setd(Varcode code, double val, int conf, const Level& lev, const Trange& tr)
 {
-    auto_ptr<Var> var(newvar(code, val));
+    unique_ptr<Var> var(newvar(code, val));
     if (conf != -1)
-        var->seta(auto_ptr<Var>(newvar(WR_VAR(0, 33, 7), conf)));
-    set(var, lev, tr);
+        var->seta(newvar(WR_VAR(0, 33, 7), conf));
+    set(std::move(var), lev, tr);
 }
 
 void Msg::setc(Varcode code, const char* val, int conf, const Level& lev, const Trange& tr)
 {
-    auto_ptr<Var> var(newvar(code, val));
+    unique_ptr<Var> var(newvar(code, val));
     if (conf != -1)
-        var->seta(auto_ptr<Var>(newvar(WR_VAR(0, 33, 7), conf)));
-    set(var, lev, tr);
-}
-
-void Msg::set_date(const char* date)
-{
-    int ye, mo, da, ho, mi, se;
-    if (sscanf(date, "%04d-%02d-%02d %02d:%02d:%02d", &ye, &mo, &da, &ho, &mi, &se) != 6)
-        error_consistency::throwf("cannot parse date/time string \"%s\"", date);
-    set_year(ye);
-    set_month(mo);
-    set_day(da);
-    set_hour(ho);
-    set_minute(mi);
-    set_second(se);
+        var->seta(newvar(WR_VAR(0, 33, 7), conf));
+    set(std::move(var), lev, tr);
 }
 
 MsgType Msg::type_from_repmemo(const char* repmemo)
@@ -673,16 +740,16 @@ void Msg::sounding_pack_levels(Msg& dst) const
         // If it is not a sounding level, just copy it
         if (ctx.find_vsig() == NULL)
         {
-            auto_ptr<msg::Context> newctx(new msg::Context(ctx));
-            dst.add_context(newctx);
+            unique_ptr<msg::Context> newctx(new msg::Context(ctx));
+            dst.add_context(std::move(newctx));
             continue;
         }
 
         // FIXME: shouldn't this also set significance bits in the output level?
         for (size_t j = 0; j < ctx.data.size(); ++j)
         {
-            auto_ptr<Var> copy(new Var(*ctx.data[j]));
-            dst.set(copy, Level(ctx.level.ltype1, ctx.level.l1), ctx.trange);
+            unique_ptr<Var> copy(new Var(*ctx.data[j]));
+            dst.set(std::move(copy), Level(ctx.level.ltype1, ctx.level.l1), ctx.trange);
         }
     }
 }
@@ -758,46 +825,6 @@ void Msg::sounding_unpack_levels(Msg& dst) const
 }
 #endif
 
-void Msg::parse_date(int* values) const
-{
-    const msg::Context* c = find_station_context();
-    if (!c)
-    {
-        for (int i = 0; i < 6; ++i)
-            values[i] = -1;
-        return;
-    }
-
-    int macros[] = { DBA_MSG_YEAR, DBA_MSG_MONTH, DBA_MSG_DAY, DBA_MSG_HOUR, DBA_MSG_MINUTE, DBA_MSG_SECOND };
-    const char* names[] = { "year", "month", "day", "hour", "minute", "second" };
-
-    for (int i = 0; i < 6; i++)
-    {
-        const wreport::Var* v = c->find_by_id(macros[i]);
-        if (v)
-            values[i] = v->enq(-1);
-        else
-            values[i] = -1;
-
-        if (i > 0 && (values[i-1] == -1 && values[i] != -1))
-            error_consistency::throwf("%s is unset but %s is set",
-                    names[i-1], names[i]);
-    }
-
-    /* Now values is either 6 times -1, 6 values, or X values followed by 6-X times -1 */
-
-    /* If one of the extremes has been selected, fill in the blanks */
-
-    if (values[0] != -1)
-    {
-        values[1] = values[1] != -1 ? values[1] : 1;
-        values[2] = values[2] != -1 ? values[2] : 1;
-        values[3] = values[3] != -1 ? values[3] : 0;
-        values[4] = values[4] != -1 ? values[4] : 0;
-        values[5] = values[5] != -1 ? values[5] : 0;
-    }
-}
-
 MatchedMsg::MatchedMsg(const Msg& m)
     : m(m)
 {
@@ -821,7 +848,7 @@ matcher::Result MatchedMsg::match_var_id(int val) const
 
 matcher::Result MatchedMsg::match_station_id(int val) const
 {
-    if (const wreport::Var* var = m.find(WR_VAR(0, 1, 192), Level::ana(), Trange::ana()))
+    if (const wreport::Var* var = m.get(WR_VAR(0, 1, 192), Level(), Trange()))
     {
         return var->enqi() == val ? matcher::MATCH_YES : matcher::MATCH_NO;
     } else
@@ -851,29 +878,28 @@ matcher::Result MatchedMsg::match_station_wmo(int block, int station) const
     return matcher::MATCH_NA;
 }
 
-matcher::Result MatchedMsg::match_date(const int* min, const int* max) const
+matcher::Result MatchedMsg::match_datetime(const DatetimeRange& range) const
 {
-    int date[6];
-    m.parse_date(date);
-    if (date[0] == -1) return matcher::MATCH_NA;
-    return Matched::date_in_range(date, min, max);
+    Datetime dt = m.get_datetime();
+    if (dt.is_missing()) return matcher::MATCH_NA;
+    return range.contains(dt) ? matcher::MATCH_YES : matcher::MATCH_NO;
 }
 
-matcher::Result MatchedMsg::match_coords(int latmin, int latmax, int lonmin, int lonmax) const
+matcher::Result MatchedMsg::match_coords(const LatRange& latrange, const LonRange& lonrange) const
 {
     const msg::Context* c = m.find_station_context();
     if (!c) return matcher::MATCH_NA;
 
     matcher::Result r1 = matcher::MATCH_NA;
     if (const wreport::Var* var = c->find_by_id(DBA_MSG_LATITUDE))
-        r1 = Matched::int_in_range(var->enqi(), latmin, latmax);
-    else if (latmin == MISSING_INT && latmax == MISSING_INT)
+        r1 = latrange.contains(var->enqi()) ? matcher::MATCH_YES : matcher::MATCH_NO;
+    else if (latrange.is_missing())
         r1 = matcher::MATCH_YES;
 
     matcher::Result r2 = matcher::MATCH_NA;
     if (const wreport::Var* var = c->find_by_id(DBA_MSG_LONGITUDE))
-        r2 = Matched::int_in_range(var->enqi(), lonmin, lonmax);
-    else if (lonmin == MISSING_INT && lonmax == MISSING_INT)
+        r2 = lonrange.contains(var->enqi()) ? matcher::MATCH_YES : matcher::MATCH_NO;
+    else if (lonrange.is_missing())
         r2 = matcher::MATCH_YES;
 
     if (r1 == matcher::MATCH_YES && r2 == matcher::MATCH_YES)
@@ -887,67 +913,66 @@ matcher::Result MatchedMsg::match_rep_memo(const char* memo) const
 {
     if (const Var* var = m.get_rep_memo_var())
     {
-        const char* val = var->value();
-        if (!val) return matcher::MATCH_NA;
-        return strcmp(memo, val) == 0 ? matcher::MATCH_YES : matcher::MATCH_NO;
+        if (!var->isset()) return matcher::MATCH_NA;
+        return strcmp(memo, var->enqc()) == 0 ? matcher::MATCH_YES : matcher::MATCH_NO;
     } else
         return matcher::MATCH_NA;
 }
 
-#if 0
 
-static dba_err dba_msg_add_context(dba_msg msg, dba_msg_context ctx)
+MatchedMessages::MatchedMessages(const Messages& m)
+    : m(m)
+{
+}
+MatchedMessages::~MatchedMessages()
 {
-    dba_err err;
-    dba_msg_context copy = NULL;
-    DBA_RUN_OR_RETURN(dba_msg_context_copy(ctx, &copy));
-    DBA_RUN_OR_GOTO(fail, dba_msg_add_context_nocopy(msg, copy));
-    return dba_error_ok();
-
-fail:
-    if (copy)
-        dba_msg_context_delete(copy);
-    return err;
 }
 
-dba_err dba_msg_set_nocopy_by_id(dba_msg msg, dba_var var, int id)
+matcher::Result MatchedMessages::match_var_id(int val) const
 {
-    dba_msg_var v = &dba_msg_vartable[id];
-    return dba_msg_set_nocopy(msg, var, v->ltype1, v->l1, v->ltype2, v->l2, v->pind, v->p1, v->p2);
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_var_id(val) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
 }
 
-dba_err dba_msg_set_by_id(dba_msg msg, dba_var var, int id)
+matcher::Result MatchedMessages::match_station_id(int val) const
 {
-    dba_msg_var v = &dba_msg_vartable[id];
-    dba_err err;
-    dba_var copy = NULL;
-
-    /* Make a copy of the variable, to give it to dba_msg_add_nocopy */
-    DBA_RUN_OR_RETURN(dba_var_create_local(v->code, &copy));
-    /* Use copy_val to ensure we get the variable code we want */
-    DBA_RUN_OR_GOTO(fail, dba_var_copy_val(copy, var));
-
-    DBA_RUN_OR_GOTO(fail, dba_msg_set_nocopy(msg, copy, v->ltype1, v->l1, v->ltype2, v->l2, v->pind, v->p1, v->p2));
-
-    return dba_error_ok();
-
-fail:
-    if (copy != NULL)
-        dba_var_delete(copy);
-    return err;
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_station_id(val) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
 }
 
-
-
-dba_msg_type dba_msg_get_type(dba_msg msg)
+matcher::Result MatchedMessages::match_station_wmo(int block, int station) const
 {
-    return msg->type;
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_station_wmo(block, station) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
 }
 
+matcher::Result MatchedMessages::match_datetime(const DatetimeRange& range) const
+{
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_datetime(range) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
+}
 
+matcher::Result MatchedMessages::match_coords(const LatRange& latrange, const LonRange& lonrange) const
+{
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_coords(latrange, lonrange) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
+}
 
-#endif
-
-} // namespace dballe
-
-/* vim:set ts=4 sw=4: */
+matcher::Result MatchedMessages::match_rep_memo(const char* memo) const
+{
+    for (const auto& i: m)
+        if (MatchedMsg(Msg::downcast(i)).match_rep_memo(memo) == matcher::MATCH_YES)
+            return matcher::MATCH_YES;
+    return matcher::MATCH_NA;
+}
+}
