@@ -1,30 +1,9 @@
-/*
- * dballe/wr_codec - BUFR/CREX import and export
- *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
 #include "wr_codec.h"
-#include "msgs.h"
+#include "msg.h"
 #include "context.h"
-#include <wreport/bulletin.h>
 #include "wr_importers/base.h"
-//#include <dballe/core/verbose.h>
+#include <wreport/bulletin.h>
+#include <wreport/vartable.h>
 
 using namespace wreport;
 using namespace std;
@@ -39,33 +18,38 @@ BufrImporter::BufrImporter(const Options& opts)
     : WRImporter(opts) {}
 BufrImporter::~BufrImporter() {}
 
-void BufrImporter::from_rawmsg(const Rawmsg& msg, Msgs& msgs) const
+bool BufrImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const
 {
-    auto_ptr<BufrBulletin> bulletin(BufrBulletin::create());
-    bulletin->decode(msg);
-    from_bulletin(*bulletin, msgs);
+    unique_ptr<BufrBulletin> bulletin(BufrBulletin::decode(msg.data));
+    return foreach_decoded_bulletin(*bulletin, dest);
 }
 
 CrexImporter::CrexImporter(const Options& opts)
     : WRImporter(opts) {}
 CrexImporter::~CrexImporter() {}
 
-void CrexImporter::from_rawmsg(const Rawmsg& msg, Msgs& msgs) const
+bool CrexImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const
 {
-    auto_ptr<CrexBulletin> bulletin(CrexBulletin::create());
-    bulletin->decode(msg);
-    from_bulletin(*bulletin, msgs);
+    unique_ptr<CrexBulletin> bulletin(CrexBulletin::decode(msg.data));
+    return foreach_decoded_bulletin(*bulletin, dest);
 }
 
-void WRImporter::from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const
+Messages WRImporter::from_bulletin(const wreport::Bulletin& msg) const
+{
+    Messages res;
+    foreach_decoded_bulletin(msg, [&](unique_ptr<Message>&& m) { res.append(move(m)); return true; });
+    return res;
+}
+
+bool WRImporter::foreach_decoded_bulletin(const wreport::Bulletin& msg, std::function<bool(std::unique_ptr<Message>&&)> dest) const
 {
     // Infer the right importer. See Common Code Table C-13
-    std::auto_ptr<wr::Importer> importer;
-    switch (msg.type)
+    std::unique_ptr<wr::Importer> importer;
+    switch (msg.data_category)
     {
         // Surface data - land
         case 0:
-            switch (msg.subtype)
+            switch (msg.data_subcategory)
             {
                 // Routine aeronautical observations (METAR)
                 case 10:
@@ -73,7 +57,7 @@ void WRImporter::from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const
                     break;
                 default:
                     // Old ECMWF METAR type
-                    if (msg.localsubtype == 140)
+                    if (msg.data_subcategory_local == 140)
                         importer = wr::Importer::createMetar(opts);
                     else
                         importer = wr::Importer::createSynop(opts);
@@ -90,7 +74,7 @@ void WRImporter::from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const
         case 4: importer = wr::Importer::createFlight(opts); break;
         // Radar data
         case 6:
-            if (msg.subtype == 1)
+            if (msg.data_subcategory == 1)
                 // Doppler wind profiles
                 importer = wr::Importer::createTemp(opts);
             else
@@ -104,11 +88,13 @@ void WRImporter::from_bulletin(const wreport::Bulletin& msg, Msgs& msgs) const
     MsgType type = importer->scanType(msg);
     for (unsigned i = 0; i < msg.subsets.size(); ++i)
     {
-        std::auto_ptr<Msg> newmsg(new Msg);
+        std::unique_ptr<Msg> newmsg(new Msg);
         newmsg->type = type;
         importer->import(msg.subsets[i], *newmsg);
-        msgs.acquire(newmsg);
+        if (!dest(move(newmsg)))
+            return false;
     }
+    return true;
 }
 
 
@@ -120,32 +106,28 @@ BufrExporter::BufrExporter(const Options& opts)
 
 BufrExporter::~BufrExporter() {}
 
-std::auto_ptr<wreport::Bulletin> BufrExporter::make_bulletin() const
+std::unique_ptr<wreport::Bulletin> BufrExporter::make_bulletin() const
 {
-    return std::auto_ptr<wreport::Bulletin>(BufrBulletin::create().release());
+    return std::unique_ptr<wreport::Bulletin>(BufrBulletin::create().release());
 }
 
-void BufrExporter::to_rawmsg(const Msgs& msgs, Rawmsg& msg) const
+std::string BufrExporter::to_binary(const Messages& msgs) const
 {
-    auto_ptr<BufrBulletin> bulletin(BufrBulletin::create());
-    to_bulletin(msgs, *bulletin);
-    bulletin->encode(msg);
+    return to_bulletin(msgs)->encode();
 }
 
 CrexExporter::CrexExporter(const Options& opts)
     : WRExporter(opts) {}
 CrexExporter::~CrexExporter() {}
 
-std::auto_ptr<wreport::Bulletin> CrexExporter::make_bulletin() const
+std::unique_ptr<wreport::Bulletin> CrexExporter::make_bulletin() const
 {
-    return std::auto_ptr<wreport::Bulletin>(CrexBulletin::create().release());
+    return std::unique_ptr<wreport::Bulletin>(CrexBulletin::create().release());
 }
 
-void CrexExporter::to_rawmsg(const Msgs& msgs, Rawmsg& msg) const
+std::string CrexExporter::to_binary(const Messages& msgs) const
 {
-    auto_ptr<CrexBulletin> bulletin(CrexBulletin::create());
-    to_bulletin(msgs, *bulletin);
-    bulletin->encode(msg);
+    return to_bulletin(msgs)->encode();
 }
 
 namespace {
@@ -162,21 +144,25 @@ const char* infer_from_message(const Msg& msg)
 
 }
 
-void WRExporter::to_bulletin(const Msgs& msgs, wreport::Bulletin& bulletin) const
+unique_ptr<wr::Template> WRExporter::infer_template(const Messages& msgs) const
 {
-    if (msgs.empty())
-        throw error_consistency("trying to export an empty message set");
-
     // Select initial template name
     string tpl = opts.template_name;
     if (tpl.empty())
-        tpl = infer_from_message(*msgs[0]);
+        tpl = infer_from_message(Msg::downcast(msgs[0]));
 
     // Get template factory
     const wr::TemplateFactory& fac = wr::TemplateRegistry::get(tpl);
-    std::auto_ptr<wr::Template> encoder = fac.make(opts, msgs);
+    return fac.factory(opts, msgs);
+}
+
+unique_ptr<Bulletin> WRExporter::to_bulletin(const Messages& msgs) const
+{
+    std::unique_ptr<wr::Template> encoder = infer_template(msgs);
     // fprintf(stderr, "Encoding with template %s\n", encoder->name());
-    encoder->to_bulletin(bulletin);
+    auto res = make_bulletin();
+    encoder->to_bulletin(*res);
+    return res;
 }
 
 namespace wr {
@@ -190,40 +176,28 @@ extern void register_flight(TemplateRegistry&);
 extern void register_generic(TemplateRegistry&);
 extern void register_pollution(TemplateRegistry&);
 
-namespace {
-struct WMOFactory : public TemplateFactory
-{
-    WMOFactory() { name = "wmo"; description = "WMO style templates (autodetect)"; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        const Msg& msg = *msgs[0];
-        string tpl;
-        switch (msg.type)
-        {
-            case MSG_TEMP_SHIP: tpl = "temp-wmo"; break;
-            default:
-                tpl = msg_type_name(msg.type);
-                tpl += "-wmo";
-                break;
-        }
-        const wr::TemplateFactory& fac = wr::TemplateRegistry::get(tpl);
-        return fac.make(opts, msgs);
-    }
-};
-}
-
 static TemplateRegistry* registry = NULL;
 const TemplateRegistry& TemplateRegistry::get()
 {
-static const TemplateFactory* wmo = NULL;
-
     if (!registry)
     {
         registry = new TemplateRegistry;
 
-        if (!wmo) wmo = new WMOFactory;
-        registry->register_factory(wmo);
+        registry->register_factory(MISSING_INT, "wmo", "WMO style templates (autodetect)",
+                [](const Exporter::Options& opts, const Messages& msgs) {
+                    const Msg& msg = Msg::downcast(msgs[0]);
+                    string tpl;
+                    switch (msg.type)
+                    {
+                        case MSG_TEMP_SHIP: tpl = "temp-wmo"; break;
+                        default:
+                            tpl = msg_type_name(msg.type);
+                            tpl += "-wmo";
+                            break;
+                    }
+                    const wr::TemplateFactory& fac = wr::TemplateRegistry::get(tpl);
+                    return fac.factory(opts, msgs);
+                });
 
         // Populate it
         register_synop(*registry);
@@ -251,12 +225,16 @@ const TemplateFactory& TemplateRegistry::get(const std::string& name)
     TemplateRegistry::const_iterator i = tr.find(name);
     if (i == tr.end())
         error_notfound::throwf("requested export template %s which does not exist", name.c_str());
-    return *(i->second);
+    return i->second;
 }
 
-void TemplateRegistry::register_factory(const TemplateFactory* fac)
+void TemplateRegistry::register_factory(
+        unsigned data_category,
+        const std::string& name,
+        const std::string& desc,
+        TemplateFactory::factory_func fac)
 {
-    insert(make_pair(fac->name, fac));
+    insert(make_pair(name, TemplateFactory(data_category, name, desc, fac)));
 }
 
 void Template::to_bulletin(wreport::Bulletin& bulletin)
@@ -266,7 +244,7 @@ void Template::to_bulletin(wreport::Bulletin& bulletin)
     for (unsigned i = 0; i < msgs.size(); ++i)
     {
         Subset& s = bulletin.obtain_subset(i);
-        to_subset(*msgs[i], s);
+        to_subset(Msg::downcast(msgs[i]), s);
     }
 }
 
@@ -274,63 +252,35 @@ void Template::setupBulletin(wreport::Bulletin& bulletin)
 {
     // Get reference time from first msg in the set
     // If not found, use current time.
-    const Msg& msg = *msgs[0];
-    bool has_date = true;
-    if (const Var* v = msg.get_year_var())
-        bulletin.rep_year = v->enqi();
-    else
-        has_date = false;
-    if (const Var* v = msg.get_month_var())
-        bulletin.rep_month = v->enqi();
-    else
-        has_date = false;
-    if (const Var* v = msg.get_day_var())
-        bulletin.rep_day = v->enqi();
-    else
-        has_date = false;
-    if (const Var* v = msg.get_hour_var())
-        bulletin.rep_hour = v->enqi();
-    else
-        bulletin.rep_hour = 0;
-    if (const Var* v = msg.get_minute_var())
-        bulletin.rep_minute = v->enqi();
-    else
-        bulletin.rep_minute = 0;
-    if (const Var* v = msg.get_second_var())
-        bulletin.rep_second = v->enqi();
-    else
-        bulletin.rep_second = 0;
+    Datetime dt = msgs[0].get_datetime();
+    bulletin.rep_year = dt.year;
+    bulletin.rep_month = dt.month;
+    bulletin.rep_day = dt.day;
+    bulletin.rep_hour = dt.hour;
+    bulletin.rep_minute = dt.minute;
+    bulletin.rep_second = dt.second;
     bulletin.master_table_number = 0;
-    if (!has_date)
-    {
-        // use today
-        time_t tnow = time(NULL);
-        struct tm now;
-        gmtime_r(&tnow, &now);
-        bulletin.rep_year = now.tm_year + 1900;
-        bulletin.rep_month = now.tm_mon + 1;
-        bulletin.rep_day = now.tm_mday;
-        bulletin.rep_hour = now.tm_hour;
-        bulletin.rep_minute = now.tm_min;
-        bulletin.rep_second = now.tm_sec;
-    }
+    bulletin.originating_centre = opts.centre != MISSING_INT ? opts.centre : 255;
+    bulletin.originating_subcentre = opts.subcentre != MISSING_INT ? opts.subcentre : 255;
+    bulletin.update_sequence_number = 0;
 
     if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
     {
         // Take from opts
-        b->centre = opts.centre != MISSING_INT ? opts.centre : 255;
-        b->subcentre = opts.subcentre != MISSING_INT ? opts.subcentre : 255;
-        b->master_table = 14;
-        b->local_table = 0;
-        b->compression = 0;
-        b->update_sequence_number = 0;
-        b->edition = 4;
+        b->edition_number = 4;
+        b->master_table_version_number = 17;
+        b->master_table_version_number_local = 0;
+        b->compression = false;
     }
     if (CrexBulletin* b = dynamic_cast<CrexBulletin*>(&bulletin))
     {
-        b->table = 3;
-        b->has_check_digit = 0;
-        b->edition = 2;
+        // TODO: change using BUFR tables, when the encoder can encode the full
+        // CREX ed.2 header
+        b->edition_number = 2;
+        b->master_table_version_number = 3;
+        b->master_table_version_number_local = 0;
+        b->master_table_version_number_bufr = 0;
+        b->has_check_digit = false;
     }
 }
 
@@ -338,7 +288,7 @@ void Template::to_subset(const Msg& msg, wreport::Subset& subset)
 {
     this->msg = &msg;
     this->subset = &subset;
-    this->c_station = msg.find_context(Level::ana(), Trange::ana());
+    this->c_station = msg.find_context(Level(), Trange());
     this->c_gnd_instant = msg.find_context(Level(1), Trange::instant());
 }
 
@@ -379,7 +329,7 @@ void Template::add(Varcode code, int shortcut) const
 
 void Template::add(Varcode code, Varcode srccode, const Level& level, const Trange& trange) const
 {
-    add(code, msg->find(srccode, level, trange));
+    add(code, msg->get(srccode, level, trange));
 }
 
 void Template::add(wreport::Varcode code, const wreport::Var* var) const
@@ -390,17 +340,23 @@ void Template::add(wreport::Varcode code, const wreport::Var* var) const
         subset->store_variable_undef(code);
 }
 
-void Template::add_st_name(wreport::Varcode dstcode, const msg::Context* ctx) const
+const Var* Template::find_station_var(wreport::Varcode code) const
 {
-    if (!ctx)
+    if (!c_station) return nullptr;
+    return c_station->find(code);
+}
+
+void Template::do_station_name(wreport::Varcode dstcode) const
+{
+    if (!c_station)
         subset->store_variable_undef(dstcode);
-    else if (const wreport::Var* var = ctx->find_by_id(DBA_MSG_ST_NAME))
+    else if (const wreport::Var* var = c_station->find_by_id(DBA_MSG_ST_NAME))
     {
-        Varinfo info = subset->btable->query(dstcode);
+        Varinfo info = subset->tables->btable->query(dstcode);
         Var name(info);
-        if (var->value())
-            name.setc_truncate(var->value());
-        subset->store_variable(name);
+        if (var->isset())
+            name.setc_truncate(var->enqc());
+        subset->store_variable(move(name));
     }
     else
         subset->store_variable_undef(dstcode);
@@ -408,9 +364,8 @@ void Template::add_st_name(wreport::Varcode dstcode, const msg::Context* ctx) co
 
 void Template::do_ecmwf_past_wtr() const
 {
-    int hour = 0;
-    if (const Var* var = msg->get_hour_var())
-        hour = var->enqi();
+    int hour = msg->get_datetime().hour == 0xff ? 0 : msg->get_datetime().hour;
+
     if (hour % 6 == 0)
     {
         add(WR_VAR(0, 20,  4), DBA_MSG_PAST_WTR1_6H);
@@ -421,6 +376,12 @@ void Template::do_ecmwf_past_wtr() const
     }
 }
 
+void Template::do_station_height() const
+{
+    add(WR_VAR(0,  7, 30), c_station);
+    add(WR_VAR(0,  7, 31), c_station);
+}
+
 void Template::do_D01001() const
 {
     add(WR_VAR(0,  1,  1), c_station, DBA_MSG_BLOCK);
@@ -430,29 +391,77 @@ void Template::do_D01001() const
 void Template::do_D01004() const
 {
     do_D01001();
-    add_st_name(WR_VAR(0, 1, 15), c_station);
+    do_station_name(WR_VAR(0, 1, 15));
     add(WR_VAR(0,  2,  1), c_station, DBA_MSG_ST_TYPE);
 }
 
 void Template::do_D01011() const
 {
-    add(WR_VAR(0,  4,  1), c_station, DBA_MSG_YEAR);
-    add(WR_VAR(0,  4,  2), c_station, DBA_MSG_MONTH);
-    add(WR_VAR(0,  4,  3), c_station, DBA_MSG_DAY);
+    // Year
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 1)))
+        subset->store_variable(WR_VAR(0, 4, 1), *var);
+    else if (!msg->get_datetime().is_missing())
+        subset->store_variable_i(WR_VAR(0, 4, 1), msg->get_datetime().year);
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 1));
+
+    // Month
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 2)))
+        subset->store_variable(WR_VAR(0, 4, 2), *var);
+    else if (!msg->get_datetime().is_missing())
+        subset->store_variable_i(WR_VAR(0, 4, 2), msg->get_datetime().month);
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 2));
+
+    // Day
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 3)))
+        subset->store_variable(WR_VAR(0, 4, 3), *var);
+    else if (!msg->get_datetime().is_missing())
+        subset->store_variable_i(WR_VAR(0, 4, 3), msg->get_datetime().day);
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 3));
 }
 
 int Template::do_D01012() const
 {
-    const Var* v_hour = c_station ? c_station->find_by_id(DBA_MSG_HOUR) : NULL;
-    add(WR_VAR(0,  4,  4), v_hour);
-    add(WR_VAR(0,  4,  5), c_station, DBA_MSG_MINUTE);
-    return v_hour ? v_hour->enqi() : MISSING_INT;
+    int res = MISSING_INT;
+
+    // Hour
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 4)))
+    {
+        subset->store_variable(WR_VAR(0, 4, 4), *var);
+        res = var->enqi();
+    }
+    else if (!msg->get_datetime().is_missing())
+    {
+        subset->store_variable_i(WR_VAR(0, 4, 4), msg->get_datetime().hour);
+        res = msg->get_datetime().hour;
+    }
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 4));
+
+    // Minute
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 5)))
+        subset->store_variable(WR_VAR(0, 4, 5), *var);
+    else if (!msg->get_datetime().is_missing())
+        subset->store_variable_i(WR_VAR(0, 4, 5), msg->get_datetime().minute);
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 5));
+
+    return res;
 }
 
 void Template::do_D01013() const
 {
     do_D01012();
-    add(WR_VAR(0,  4,  6), c_station, DBA_MSG_SECOND);
+
+    // Second
+    if (const Var* var = find_station_var(WR_VAR(0, 4, 6)))
+        subset->store_variable(WR_VAR(0, 4, 6), *var);
+    else if (!msg->get_datetime().is_missing())
+        subset->store_variable_i(WR_VAR(0, 4, 6), msg->get_datetime().second);
+    else
+        subset->store_variable_undef(WR_VAR(0, 4, 6));
 }
 
 void Template::do_D01021() const
@@ -467,10 +476,12 @@ void Template::do_D01022() const
     add(WR_VAR(0,  7,  1), c_station, DBA_MSG_HEIGHT_STATION);
 }
 
-} // namespace wr
+void Template::do_D01023() const
+{
+    add(WR_VAR(0,  5,  2), c_station, DBA_MSG_LATITUDE);
+    add(WR_VAR(0,  6,  2), c_station, DBA_MSG_LONGITUDE);
+}
 
-
-} // namespace msg
-} // namespace dballe
-
-/* vim:set ts=4 sw=4: */
+}
+}
+}
