@@ -1,7 +1,7 @@
 /*
  * dballe/wr_exporter/common - Common infrastructure for wreport exporters
  *
- * Copyright (C) 2013--2014  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2013--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,9 @@ static int override_trange(const Var* var, int orig)
     return orig;
 }
 
-void ExporterModule::init(wreport::Subset& subset)
+void ExporterModule::init(const Msg& msg, wreport::Subset& subset)
 {
+    this->msg = &msg;
     this->subset = &subset;
     c_surface_instant = 0;
     c_ana = 0;
@@ -99,66 +100,6 @@ void ExporterModule::add(wreport::Varcode code, const wreport::Var* var) const
         subset->store_variable_undef(code);
 }
 
-int ExporterModule::get_hour()
-{
-    if (!c_ana) return MISSING_INT;
-    const Var* v_hour = c_ana->find_by_id(DBA_MSG_HOUR);
-    return v_hour ? v_hour->enqi() : MISSING_INT;
-}
-
-void ExporterModule::add_year_to_minute()
-{
-    add(WR_VAR(0,  4,  1), c_ana);
-    add(WR_VAR(0,  4,  2), c_ana);
-    add(WR_VAR(0,  4,  3), c_ana);
-    add(WR_VAR(0,  4,  4), c_ana);
-    add(WR_VAR(0,  4,  5), c_ana);
-}
-
-void ExporterModule::add_latlon_coarse()
-{
-    add(WR_VAR(0,  5,  2), c_ana, DBA_MSG_LATITUDE);
-    add(WR_VAR(0,  6,  2), c_ana, DBA_MSG_LONGITUDE);
-}
-
-void ExporterModule::add_latlon_high()
-{
-    add(WR_VAR(0,  5,  1), c_ana, DBA_MSG_LATITUDE);
-    add(WR_VAR(0,  6,  1), c_ana, DBA_MSG_LONGITUDE);
-}
-
-void ExporterModule::add_station_height()
-{
-    add(WR_VAR(0,  7, 30), c_ana);
-    add(WR_VAR(0,  7, 31), c_ana);
-}
-
-void ExporterModule::add_station_name(wreport::Varcode code)
-{
-    // Append an undefined station name
-    subset->store_variable_undef(code);
-
-    if (!c_ana) return;
-
-    if (const wreport::Var* var = c_ana->find_by_id(DBA_MSG_ST_NAME))
-    {
-        // Add the value with truncation
-        if (var->value())
-            (*subset)[subset->size() - 1].setc_truncate(var->value());
-    }
-}
-
-void ExporterModule::add_D01090()
-{
-    add(WR_VAR(0,  1,  1), c_ana);
-    add(WR_VAR(0,  1,  2), c_ana);
-    add_station_name(WR_VAR(0,  1, 15));
-    add(WR_VAR(0,  2,  1), c_ana);
-    add_year_to_minute();
-    add_latlon_high();
-    add_station_height();
-}
-
 void ExporterModule::add_ecmwf_synop_head()
 {
     add(WR_VAR(0,  1,  1), c_ana);
@@ -166,25 +107,9 @@ void ExporterModule::add_ecmwf_synop_head()
     add(WR_VAR(0,  2,  1), c_ana);
 }
 
-void ExporterModule::add_ship_head()
+void CommonSynopExporter::init(const Msg& msg, wreport::Subset& subset)
 {
-    add(WR_VAR(0,  1, 11), c_ana);
-    add(WR_VAR(0,  1, 12), c_surface_instant);
-    add(WR_VAR(0,  1, 13), c_surface_instant);
-    add(WR_VAR(0,  2,  1), c_ana);
-}
-
-void ExporterModule::add_D01093()
-{
-    add_ship_head();
-    add_year_to_minute();
-    add_latlon_coarse();
-    add_station_height();
-}
-
-void CommonSynopExporter::init(wreport::Subset& subset)
-{
-    ExporterModule::init(subset);
+    ExporterModule::init(msg, subset);
     c_geopotential = 0;
     c_thermo = 0;
     c_tmax = 0;
@@ -387,7 +312,12 @@ void CommonSynopExporter::add_D02031()
 void CommonSynopExporter::add_pressure()
 {
     if (c_geopotential)
-        subset->store_variable_d(WR_VAR(0, 7, 4), c_geopotential->level.l1);
+    {
+        if (c_geopotential->level.l1 == MISSING_INT)
+            subset->store_variable_undef(WR_VAR(0, 7, 4));
+        else
+            subset->store_variable_d(WR_VAR(0, 7, 4), c_geopotential->level.l1);
+    }
     else
         subset->store_variable_undef(WR_VAR(0,  7,  4));
 }
@@ -796,7 +726,7 @@ void CommonSynopExporter::add_D02038()
     add(WR_VAR(0, 20,  3), c_surface_instant);
     if (c_past_wtr)
     {
-        int hour = get_hour();
+        int hour = msg->get_datetime().hour;
         // Look for a p2 override in the attributes
         const Var* v = c_past_wtr->find(WR_VAR(0, 20, 4));
         if (!v) v = c_past_wtr->find(WR_VAR(0, 20, 5));
