@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
  */
 
 #include "msg/wr_codec.h"
-#include <wreport/bulletin.h>
-#include "msg/msgs.h"
+#include "msg/msg.h"
 #include "msg/context.h"
+#include <wreport/bulletin.h>
 
 using namespace wreport;
 using namespace std;
@@ -61,7 +61,7 @@ struct FlightBase : public Template
     bool is_crex;
     const msg::Context* flight_ctx;
 
-    FlightBase(const Exporter::Options& opts, const Msgs& msgs)
+    FlightBase(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs), flight_ctx(0) {}
 
     void add(wreport::Varcode code, const wreport::Var* var) const
@@ -99,13 +99,13 @@ struct FlightBase : public Template
         // Use old table for old templates
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->master_table = 13;
+            b->master_table_version_number = 13;
         }
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
-        bulletin.type = 4;
-        bulletin.subtype = 255;
+        bulletin.data_category = 4;
+        bulletin.data_subcategory = 255;
     }
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
     {
@@ -146,7 +146,7 @@ struct FlightBase : public Template
 
 struct Airep : public FlightBase
 {
-    Airep(const Exporter::Options& opts, const Msgs& msgs)
+    Airep(const Exporter::Options& opts, const Messages& msgs)
         : FlightBase(opts, msgs) {}
 
     virtual const char* name() const { return AIREP_NAME; }
@@ -155,7 +155,7 @@ struct Airep : public FlightBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         FlightBase::setupBulletin(bulletin);
-        bulletin.localsubtype = 142;
+        bulletin.data_subcategory_local = 142;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -180,11 +180,8 @@ struct Airep : public FlightBase
 
         /*  0 */ add(WR_VAR(0,  1,  6), DBA_MSG_IDENT);
         /*  1 */ add(WR_VAR(0,  2, 61));
-        /*  2 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /*  3 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /*  4 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /*  5 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /*  6 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        do_D01011();
+        do_D01012();
         /*  7 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
         /*  8 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
         /*  9 */ add(WR_VAR(0,  8,  4));
@@ -214,7 +211,7 @@ struct Airep : public FlightBase
 
 struct Amdar : public Airep
 {
-    Amdar(const Exporter::Options& opts, const Msgs& msgs)
+    Amdar(const Exporter::Options& opts, const Messages& msgs)
         : Airep(opts, msgs) {}
 
     virtual const char* name() const { return AMDAR_NAME; }
@@ -223,13 +220,13 @@ struct Amdar : public Airep
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         Airep::setupBulletin(bulletin);
-        bulletin.localsubtype = 144;
+        bulletin.data_subcategory_local = 144;
     }
 };
 
 struct AmdarWMO : public FlightBase
 {
-    AmdarWMO(const Exporter::Options& opts, const Msgs& msgs)
+    AmdarWMO(const Exporter::Options& opts, const Messages& msgs)
         : FlightBase(opts, msgs) {}
 
     virtual const char* name() const { return AMDAR_WMO_NAME; }
@@ -238,8 +235,8 @@ struct AmdarWMO : public FlightBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         FlightBase::setupBulletin(bulletin);
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 144;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 144;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -284,12 +281,8 @@ struct AmdarWMO : public FlightBase
         /*  3 */ add(WR_VAR(0,  1, 23));
         /*  4 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
         /*  5 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
-        /*  6 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /*  7 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /*  8 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /*  9 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /* 10 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
-        /* 11 */ add(WR_VAR(0,  4,  6), DBA_MSG_SECOND);
+        do_D01011();
+        do_D01013();
         /* 12 */
         if (const wreport::Var* v = flight_ctx->find(WR_VAR(0,  7, 30)))
             add(WR_VAR(0,  7, 10), v);
@@ -332,7 +325,7 @@ struct AmdarWMO : public FlightBase
 
 struct Acars : public FlightBase
 {
-    Acars(const Exporter::Options& opts, const Msgs& msgs)
+    Acars(const Exporter::Options& opts, const Messages& msgs)
         : FlightBase(opts, msgs) {}
 
     virtual const char* name() const { return ACARS_NAME; }
@@ -342,7 +335,7 @@ struct Acars : public FlightBase
     {
         FlightBase::setupBulletin(bulletin);
 
-        bulletin.localsubtype = 145;
+        bulletin.data_subcategory_local = 145;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -396,11 +389,8 @@ struct Acars : public FlightBase
         /*  6 */ add(WR_VAR(0,  2, 70));
         /*  7 */ add(WR_VAR(0,  2, 63));
         /*  8 */ add(WR_VAR(0,  2,  1));
-        /*  9 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /* 10 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /* 11 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /* 12 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /* 13 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        do_D01011();
+        do_D01012();
         /* 14 */ add(WR_VAR(0,  5,  2), DBA_MSG_LATITUDE);
         /* 15 */ add(WR_VAR(0,  6,  2), DBA_MSG_LONGITUDE);
         /* 16 */ add(WR_VAR(0,  8,  4));
@@ -433,7 +423,7 @@ struct Acars : public FlightBase
 
 struct AcarsWMO : public AmdarWMO
 {
-    AcarsWMO(const Exporter::Options& opts, const Msgs& msgs)
+    AcarsWMO(const Exporter::Options& opts, const Messages& msgs)
         : AmdarWMO(opts, msgs) {}
 
     virtual const char* name() const { return ACARS_WMO_NAME; }
@@ -443,118 +433,50 @@ struct AcarsWMO : public AmdarWMO
     {
         AmdarWMO::setupBulletin(bulletin);
 
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 145;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 145;
     }
 };
 
-struct AirepFactory : public TemplateFactory
-{
-    AirepFactory() { name = AIREP_NAME; description = AIREP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Airep(opts, msgs));
-    }
-};
-struct AirepEcmwfFactory : public TemplateFactory
-{
-    AirepEcmwfFactory() { name = AIREP_ECMWF_NAME; description = AIREP_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Airep(opts, msgs));
-    }
-};
-struct AmdarFactory : public TemplateFactory
-{
-    AmdarFactory() { name = AMDAR_NAME; description = AMDAR_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Amdar(opts, msgs));
-    }
-};
-struct AmdarEcmwfFactory : public TemplateFactory
-{
-    AmdarEcmwfFactory() { name = AMDAR_ECMWF_NAME; description = AMDAR_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Amdar(opts, msgs));
-    }
-};
-struct AmdarWMOFactory : public TemplateFactory
-{
-    AmdarWMOFactory() { name = AMDAR_WMO_NAME; description = AMDAR_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new AmdarWMO(opts, msgs));
-    }
-};
-struct AcarsFactory : public TemplateFactory
-{
-    AcarsFactory() { name = ACARS_NAME; description = ACARS_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Acars(opts, msgs));
-    }
-};
-struct AcarsEcmwfFactory : public TemplateFactory
-{
-    AcarsEcmwfFactory() { name = ACARS_ECMWF_NAME; description = ACARS_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Acars(opts, msgs));
-    }
-};
-struct AcarsWMOFactory : public TemplateFactory
-{
-    AcarsWMOFactory() { name = ACARS_WMO_NAME; description = ACARS_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new AcarsWMO(opts, msgs));
-    }
-};
 
 } // anonymous namespace
 
 void register_flight(TemplateRegistry& r)
 {
-static const TemplateFactory* airep = NULL;
-static const TemplateFactory* airepecmwf = NULL;
-static const TemplateFactory* amdar = NULL;
-static const TemplateFactory* amdarecmwf = NULL;
-static const TemplateFactory* amdarwmo = NULL;
-static const TemplateFactory* acars = NULL;
-static const TemplateFactory* acarsecmwf = NULL;
-static const TemplateFactory* acarswmo = NULL;
-
-    if (!airep) airep = new AirepFactory;
-    if (!airepecmwf) airepecmwf = new AirepEcmwfFactory;
-    if (!amdar) amdar = new AmdarFactory;
-    if (!amdarecmwf) amdarecmwf = new AmdarEcmwfFactory;
-    if (!amdarwmo) amdarwmo = new AmdarWMOFactory;
-    if (!acars) acars = new AcarsFactory;
-    if (!acarsecmwf) acarsecmwf = new AcarsEcmwfFactory;
-    if (!acarswmo) acarswmo = new AcarsWMOFactory;
-
-    r.register_factory(airep);
-    r.register_factory(airepecmwf);
-    r.register_factory(amdar);
-    r.register_factory(amdarecmwf);
-    r.register_factory(amdarwmo);
-    r.register_factory(acars);
-    r.register_factory(acarsecmwf);
-    r.register_factory(acarswmo);
+    r.register_factory(4, AIREP_NAME, AIREP_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Airep(opts, msgs));
+            });
+    r.register_factory(4, AIREP_ECMWF_NAME, AIREP_ECMWF_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Airep(opts, msgs));
+            });
+    r.register_factory(4, AMDAR_NAME, AMDAR_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Amdar(opts, msgs));
+            });
+    r.register_factory(4, AMDAR_ECMWF_NAME, AMDAR_ECMWF_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Amdar(opts, msgs));
+            });
+    r.register_factory(4, AMDAR_WMO_NAME, AMDAR_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new AmdarWMO(opts, msgs));
+            });
+    r.register_factory(4, ACARS_NAME, ACARS_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Acars(opts, msgs));
+            });
+    r.register_factory(4, ACARS_ECMWF_NAME, ACARS_ECMWF_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Acars(opts, msgs));
+            });
+    r.register_factory(4, ACARS_WMO_NAME, ACARS_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new AcarsWMO(opts, msgs));
+            });
 }
 
 }
 }
 }
-
-/* vim:set ts=4 sw=4: */
