@@ -18,9 +18,9 @@
  */
 
 #include "msg/wr_codec.h"
-#include <wreport/bulletin.h>
-#include "msg/msgs.h"
+#include "msg/msg.h"
 #include "msg/context.h"
+#include <wreport/bulletin.h>
 #include <cstdlib>
 
 using namespace wreport;
@@ -39,7 +39,7 @@ struct Generic : public Template
 {
     Bulletin* bulletin;
 
-    Generic(const Exporter::Options& opts, const Msgs& msgs)
+    Generic(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     virtual const char* name() const { return GENERIC_NAME; }
@@ -56,31 +56,60 @@ struct Generic : public Template
 
     void add(Varcode code, Varcode srccode, const Level& level, const Trange& trange)
     {
-        const Var* var = msg->find(srccode, level, trange);
+        const Var* var = msg->get(srccode, level, trange);
         if (var)
             subset->store_variable(code, *var);
         else
             subset->store_variable_undef(code);
     }
 
+    void add_var_and_attrs(const Var& var)
+    {
+        // Store the variable
+        subset->store_variable(var.code(), var);
+
+        // Store the attributes
+        for (const Var* attr = var.next_attr(); attr != NULL; attr = attr->next_attr())
+        {
+            if (WR_VAR_X(attr->code()) != 33)
+                // Skip non-B33yyy attributes, as they won't be decoded properly
+                if (WR_VAR_X(attr->code()) != 33)
+                    continue;
+                    //error_consistency::throwf("attempt to encode attribute B%02d%03d which is not B33YYY",
+                    //        WR_VAR_X(attr->code()), WR_VAR_Y(attr->code()));
+            subset->store_variable(attr->code(), *attr);
+        }
+    }
+
+    void add_datetime_var(Varcode code, bool has_default, int def)
+    {
+        if (const Var* var = find_station_var(code))
+            add_var_and_attrs(*var);
+        else if (has_default)
+            subset->store_variable_i(code, def);
+        else
+            subset->store_variable_undef(code);
+    }
+
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         Template::setupBulletin(bulletin);
 
-        bulletin.type = 255;
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 0;
+        bulletin.data_category = 255;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 0;
+        bulletin.originating_centre = 200;
+        bulletin.originating_subcentre = 0;
 
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->centre = 200;
-            b->subcentre = 0;
-            b->master_table = 14;
-            b->local_table = 1;
+            b->master_table_version_number = 14;
+            b->master_table_version_number_local = 1;
         }
         if (CrexBulletin* b = dynamic_cast<CrexBulletin*>(&bulletin))
         {
-            b->table = 99;
+            // TODO: change to BUFR-like info when we can encode BUFR table info in Crex
+            b->master_table_version_number = 99;
         }
 
         // The data descriptor section will be generated later, as it depends
@@ -99,35 +128,49 @@ struct Generic : public Template
         Level lev;
         Trange tr;
 
-        // Do the report type first
+        // Report type
         if (repmemo)
             subset.store_variable(repmemo->code(), *repmemo);
         else if (msg.type != MSG_GENERIC) // It is generic by default, no need to repeat it
             subset.store_variable_c(WR_VAR(0, 1, 194), Msg::repmemo_from_type(msg.type));
+        else
+            subset.store_variable_undef(WR_VAR(0, 1, 194));
+
+        // Datetime
+        Datetime dt = msg.get_datetime();
+        add_datetime_var(WR_VAR(0, 4, 1), !dt.is_missing(), dt.year);
+        add_datetime_var(WR_VAR(0, 4, 2), !dt.is_missing(), dt.month);
+        add_datetime_var(WR_VAR(0, 4, 3), !dt.is_missing(), dt.day);
+        add_datetime_var(WR_VAR(0, 4, 4), !dt.is_missing(), dt.hour);
+        add_datetime_var(WR_VAR(0, 4, 5), !dt.is_missing(), dt.minute);
+        add_datetime_var(WR_VAR(0, 4, 6), !dt.is_missing(), dt.second);
 
         // Then the station context
-        subset.store_variable_i(WR_VAR(0, 7, 192), 257);
         if (const msg::Context* ctx = msg.find_station_context())
+        {
             for (size_t j = 0; j < ctx->data.size(); ++j)
             {
                 const Var& var = *(ctx->data[j]);
 
-                // Do not add rep_memo twice
-                if (var.code() == WR_VAR(0, 1, 194))
-                    continue;
+                // Do not add rep_memo and datetime twice
+                switch (var.code())
+                {
+                    case WR_VAR(0, 1, 194):
+                    case WR_VAR(0, 4, 1):
+                    case WR_VAR(0, 4, 2):
+                    case WR_VAR(0, 4, 3):
+                    case WR_VAR(0, 4, 4):
+                    case WR_VAR(0, 4, 5):
+                    case WR_VAR(0, 4, 6):
+                        continue;
+                    default:
+                        break;
+                }
 
                 // Store the variable
-                subset.store_variable(var.code(), var);
-
-                // Store the attributes
-                for (const Var* attr = var.next_attr(); attr != NULL; attr = attr->next_attr())
-                {
-                    if (WR_VAR_X(attr->code()) != 33)
-                        error_consistency::throwf("attempt to encode attribute B%02d%03d which is not B33YYY",
-                                WR_VAR_X(attr->code()), WR_VAR_Y(attr->code()));
-                    subset.store_variable(attr->code(), *attr);
-                }
+                add_var_and_attrs(var);
             }
+        }
 
         // Then do the other contexts
         for (size_t i = 0; i < msg.data.size(); ++i)
@@ -138,7 +181,7 @@ struct Generic : public Template
             for (size_t j = 0; j < ctx.data.size(); ++j)
             {
                 const Var& var = *ctx.data[j];
-                if (var.value() == NULL) continue; // Don't add undef vars
+                if (!var.isset()) continue; // Don't add undef vars
                 if (&var == repmemo) continue; // Don't add rep_memo again
 
                 /* Update the context in the message, if needed */
@@ -200,18 +243,7 @@ struct Generic : public Template
                 }
 
                 // Store the variable
-                subset.store_variable(var.code(), var);
-
-                // Store the attributes
-                for (const Var* attr = var.next_attr(); attr != NULL; attr = attr->next_attr())
-                {
-                    // Skip non-B33yyy attributes, as they won't be decoded properly
-                    if (WR_VAR_X(attr->code()) != 33)
-                        continue;
-                        //error_consistency::throwf("attempt to encode attribute B%02d%03d which is not B33YYY",
-                        //        WR_VAR_X(attr->code()), WR_VAR_Y(attr->code()));
-                    subset.store_variable(attr->code(), *attr);
-                }
+                add_var_and_attrs(var);
             }
         }
 
@@ -222,29 +254,16 @@ struct Generic : public Template
     }
 };
 
-struct GenericFactory : public TemplateFactory
-{
-    GenericFactory() { name = GENERIC_NAME; description = GENERIC_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Generic(opts, msgs));
-    }
-};
-
 } // anonymous namespace
 
 void register_generic(TemplateRegistry& r)
 {
-static const TemplateFactory* generic = NULL;
-
-    if (!generic) generic = new GenericFactory;
-
-    r.register_factory(generic);
+    r.register_factory(255, GENERIC_NAME, GENERIC_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Generic(opts, msgs));
+            });
 }
 
 }
 }
 }
-
-/* vim:set ts=4 sw=4: */
