@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
  */
 
 #include "msg/wr_codec.h"
-#include <wreport/bulletin.h>
-#include "msg/msgs.h"
+#include "msg/msg.h"
 #include "msg/context.h"
+#include <wreport/bulletin.h>
 #include <cstdlib>
 #include <cmath>
 
@@ -64,7 +64,7 @@ static double intexp10(unsigned x)
 
 struct Pollution : public Template
 {
-    Pollution(const Exporter::Options& opts, const Msgs& msgs)
+    Pollution(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     virtual const char* name() const { return POLLUTION_NAME; }
@@ -81,7 +81,7 @@ struct Pollution : public Template
 
     void add(Varcode code, Varcode srccode, const Level& level, const Trange& trange)
     {
-        const Var* var = msg->find(srccode, level, trange);
+        const Var* var = msg->get(srccode, level, trange);
         if (var)
             subset->store_variable(code, *var);
         else
@@ -92,16 +92,16 @@ struct Pollution : public Template
     {
         Template::setupBulletin(bulletin);
 
-        bulletin.type = 8;
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 171;
+        bulletin.data_category = 8;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 171;
+        bulletin.originating_centre = 98;
+        bulletin.originating_subcentre = 0;
 
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->centre = 98;
-            b->subcentre = 0;
-            b->master_table = 13;
-            b->local_table = 102;
+            b->master_table_version_number = 13;
+            b->master_table_version_number_local = 102;
         }
 
         // Data descriptor section
@@ -222,12 +222,8 @@ struct Pollution : public Template
         /*  4 */ add(WR_VAR(0,  1, 215), DBA_MSG_POLL_SOURCE);
         /*  5 */ add(WR_VAR(0,  1, 216), DBA_MSG_POLL_ATYPE);
         /*  6 */ add(WR_VAR(0,  1, 217), DBA_MSG_POLL_TTYPE);
-        /*  7 */ add(WR_VAR(0,  4,   1), DBA_MSG_YEAR);
-        /*  8 */ add(WR_VAR(0,  4,   2), DBA_MSG_MONTH);
-        /*  9 */ add(WR_VAR(0,  4,   3), DBA_MSG_DAY);
-        /* 10 */ add(WR_VAR(0,  4,   4), DBA_MSG_HOUR);
-        /* 11 */ add(WR_VAR(0,  4,   5), DBA_MSG_MINUTE);
-        /* 12 */ add(WR_VAR(0,  4,   6), DBA_MSG_SECOND);
+        do_D01011();
+        do_D01013();
         /* 13 */ add(WR_VAR(0,  5,   1), DBA_MSG_LATITUDE);
         /* 14 */ add(WR_VAR(0,  6,   1), DBA_MSG_LONGITUDE);
         /* 15 */ add(WR_VAR(0,  7,  30), DBA_MSG_HEIGHT_STATION);
@@ -257,29 +253,16 @@ struct Pollution : public Template
     }
 };
 
-struct PollutionFactory : public TemplateFactory
-{
-    PollutionFactory() { name = POLLUTION_NAME; description = POLLUTION_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new Pollution(opts, msgs));
-    }
-};
-
 } // anonymous namespace
 
 void register_pollution(TemplateRegistry& r)
 {
-static const TemplateFactory* pollution = NULL;
-
-    if (!pollution) pollution = new PollutionFactory;
- 
-    r.register_factory(pollution);
+    r.register_factory(8, POLLUTION_NAME, POLLUTION_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new Pollution(opts, msgs));
+            });
 }
 
 }
 }
 }
-
-/* vim:set ts=4 sw=4: */
