@@ -20,15 +20,11 @@
 #include "common.h"
 #include "msg/wr_codec.h"
 #include <wreport/bulletin.h>
-#include "msg/msgs.h"
 #include "msg/context.h"
 
 using namespace wreport;
 using namespace std;
 
-#define SHIP_NAME "ship"
-#define SHIP_DESC "Synop ship (autodetect)"
-
 #define SHIP_PLAIN_NAME "ship-plain"
 #define SHIP_PLAIN_DESC "Synop ship (normal) (1.11)"
 
@@ -59,7 +55,7 @@ struct ShipBase : public Template
     CommonSynopExporter synop;
     bool is_crex;
 
-    ShipBase(const Exporter::Options& opts, const Msgs& msgs)
+    ShipBase(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     virtual void setupBulletin(wreport::Bulletin& bulletin)
@@ -68,24 +64,41 @@ struct ShipBase : public Template
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
-        bulletin.type = 1;
-        bulletin.subtype = 255;
+        bulletin.data_category = 1;
+        bulletin.data_subcategory = 255;
     }
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
     {
         Template::to_subset(msg, subset);
-        synop.init(subset);
+        synop.init(msg, subset);
 
         // Scan message finding context for the data that follow
         for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
                 i != msg.data.end(); ++i)
             synop.scan_context(**i);
     }
+
+    void do_D01093()
+    {
+        do_ship_head();
+        do_D01011();
+        do_D01012();
+        do_D01023();
+        do_station_height();
+    }
+
+    void do_ship_head()
+    {
+        add(WR_VAR(0,  1, 11), c_station);
+        add(WR_VAR(0,  1, 12), c_gnd_instant);
+        add(WR_VAR(0,  1, 13), c_gnd_instant);
+        add(WR_VAR(0,  2,  1), c_station);
+    }
 };
 
 struct ShipECMWFBase : public ShipBase
 {
-    ShipECMWFBase(const Exporter::Options& opts, const Msgs& msgs)
+    ShipECMWFBase(const Exporter::Options& opts, const Messages& msgs)
         : ShipBase(opts, msgs) {}
 
     virtual void setupBulletin(wreport::Bulletin& bulletin)
@@ -95,11 +108,11 @@ struct ShipECMWFBase : public ShipBase
         // Use old table for old templates
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->master_table = 13;
+            b->master_table_version_number = 13;
         }
 
-        bulletin.type = 1;
-        bulletin.subtype = 255;
+        bulletin.data_category = 1;
+        bulletin.data_subcategory = 255;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -108,13 +121,13 @@ struct ShipECMWFBase : public ShipBase
         bulletin.datadesc.push_back(WR_VAR(0, 10, 197));
         if (!is_crex)
         {
-                bulletin.datadesc.push_back(WR_VAR(2, 22,   0));
-                bulletin.datadesc.push_back(WR_VAR(1,  1,  34));
-                bulletin.datadesc.push_back(WR_VAR(0, 31,  31));
-                bulletin.datadesc.push_back(WR_VAR(0,  1,  31));
-                bulletin.datadesc.push_back(WR_VAR(0,  1,  32));
-                bulletin.datadesc.push_back(WR_VAR(1,  1,  34));
-                bulletin.datadesc.push_back(WR_VAR(0, 33,   7));
+            bulletin.datadesc.push_back(WR_VAR(2, 22,   0));
+            bulletin.datadesc.push_back(WR_VAR(1,  1,  34));
+            bulletin.datadesc.push_back(WR_VAR(0, 31,  31));
+            bulletin.datadesc.push_back(WR_VAR(0,  1,  31));
+            bulletin.datadesc.push_back(WR_VAR(0,  1,  32));
+            bulletin.datadesc.push_back(WR_VAR(1,  1,  34));
+            bulletin.datadesc.push_back(WR_VAR(0, 33,   7));
         }
     }
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
@@ -131,9 +144,10 @@ struct ShipECMWFBase : public ShipBase
                 c_wind = c;
         }
 
-        synop.add_ship_head();
-        synop.add_year_to_minute();
-        synop.add_latlon_coarse();
+        do_ship_head();
+        do_D01011();
+        do_D01012();
+        do_D01023();
         /* 11 */ add(WR_VAR(0, 10,  4), DBA_MSG_PRESS);
         /* 12 */ add(WR_VAR(0, 10, 51), DBA_MSG_PRESS_MSL);
         /* 13 */ add(WR_VAR(0, 10, 61), DBA_MSG_PRESS_3H);
@@ -174,7 +188,7 @@ struct ShipECMWFBase : public ShipBase
 
 struct ShipAbbr : public ShipECMWFBase
 {
-    ShipAbbr(const Exporter::Options& opts, const Msgs& msgs)
+    ShipAbbr(const Exporter::Options& opts, const Messages& msgs)
         : ShipECMWFBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_ABBR_NAME; }
@@ -183,7 +197,7 @@ struct ShipAbbr : public ShipECMWFBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         ShipECMWFBase::setupBulletin(bulletin);
-        bulletin.localsubtype = 9;
+        bulletin.data_subcategory_local = 9;
 
         bulletin.load_tables();
     }
@@ -191,7 +205,7 @@ struct ShipAbbr : public ShipECMWFBase
 
 struct ShipPlain : public ShipECMWFBase
 {
-    ShipPlain(const Exporter::Options& opts, const Msgs& msgs)
+    ShipPlain(const Exporter::Options& opts, const Messages& msgs)
         : ShipECMWFBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_PLAIN_NAME; }
@@ -200,7 +214,7 @@ struct ShipPlain : public ShipECMWFBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         ShipECMWFBase::setupBulletin(bulletin);
-        bulletin.localsubtype = 11;
+        bulletin.data_subcategory_local = 11;
 
         bulletin.load_tables();
     }
@@ -208,7 +222,7 @@ struct ShipPlain : public ShipECMWFBase
 
 struct ShipAuto : public ShipECMWFBase
 {
-    ShipAuto(const Exporter::Options& opts, const Msgs& msgs)
+    ShipAuto(const Exporter::Options& opts, const Messages& msgs)
         : ShipECMWFBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_AUTO_NAME; }
@@ -217,7 +231,7 @@ struct ShipAuto : public ShipECMWFBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         ShipECMWFBase::setupBulletin(bulletin);
-        bulletin.localsubtype = 13;
+        bulletin.data_subcategory_local = 13;
 
         bulletin.load_tables();
     }
@@ -225,7 +239,7 @@ struct ShipAuto : public ShipECMWFBase
 
 struct ShipReduced : public ShipECMWFBase
 {
-    ShipReduced(const Exporter::Options& opts, const Msgs& msgs)
+    ShipReduced(const Exporter::Options& opts, const Messages& msgs)
         : ShipECMWFBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_REDUCED_NAME; }
@@ -234,7 +248,7 @@ struct ShipReduced : public ShipECMWFBase
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
         ShipECMWFBase::setupBulletin(bulletin);
-        bulletin.localsubtype = 19;
+        bulletin.data_subcategory_local = 19;
 
         bulletin.load_tables();
     }
@@ -242,7 +256,7 @@ struct ShipReduced : public ShipECMWFBase
 
 struct ShipECMWFSecondRecord : public ShipBase
 {
-    ShipECMWFSecondRecord(const Exporter::Options& opts, const Msgs& msgs)
+    ShipECMWFSecondRecord(const Exporter::Options& opts, const Messages& msgs)
         : ShipBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_ECMWF_SECOND_NAME; }
@@ -255,12 +269,12 @@ struct ShipECMWFSecondRecord : public ShipBase
         // Use old table for old templates
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->master_table = 13;
+            b->master_table_version_number = 13;
         }
 
-        bulletin.type = 1;
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 12;
+        bulletin.data_category = 1;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 12;
         bulletin.load_tables();
 
         // Data descriptor section
@@ -301,9 +315,10 @@ struct ShipECMWFSecondRecord : public ShipBase
     {
         ShipBase::to_subset(msg, subset);
 
-        synop.add_ship_head();
-        synop.add_year_to_minute();
-        synop.add_latlon_coarse();
+        do_ship_head();
+        do_D01011();
+        do_D01012();
+        do_D01023();
 
         // TODO
         subset.store_variable_undef(WR_VAR(0, 12,  15)); // MINIMUM TEMPERATURE AT 2M, PAST 12 HOURS
@@ -340,7 +355,7 @@ struct ShipWMO : public ShipBase
 {
     bool is_crex;
 
-    ShipWMO(const Exporter::Options& opts, const Msgs& msgs)
+    ShipWMO(const Exporter::Options& opts, const Messages& msgs)
         : ShipBase(opts, msgs) {}
 
     virtual const char* name() const { return SHIP_WMO_NAME; }
@@ -352,9 +367,9 @@ struct ShipWMO : public ShipBase
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
-        bulletin.type = 1;
-        bulletin.subtype = 0;
-        bulletin.localsubtype = 255;
+        bulletin.data_category = 1;
+        bulletin.data_subcategory = 0;
+        bulletin.data_subcategory_local = 255;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -378,7 +393,7 @@ struct ShipWMO : public ShipBase
 
         // Ship identification, movement, date/time, horizontal and vertical
         // coordinates
-        synop.add_D01093();
+        do_D01093();
 
         // Pressure data
         synop.add_D02001();
@@ -416,145 +431,90 @@ struct ShipWMO : public ShipBase
 };
 
 
-struct ShipFactory : public TemplateFactory
-{
-    ShipFactory() { name = SHIP_NAME; description = SHIP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        // Scan msgs and pick the right one
-        bool maybe_wmo = true;
-        bool maybe_plain = true;
-        bool maybe_auto = true;
-        bool maybe_second = true;
-        const Msg& msg = *msgs[0];
-        for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
-                i != msg.data.end(); ++i)
-        {
-            const msg::Context& c = **i;
-            switch (c.level.ltype1)
-            {
-                case MISSING_INT:
-                    for (std::vector<wreport::Var*>::const_iterator vi = c.data.begin();
-                            vi != c.data.end(); ++vi)
-                    {
-                        switch ((*vi)->code())
-                        {
-                            case WR_VAR(0, 2, 1):
-                                switch ((*vi)->enq(0))
-                                {
-                                    case 0: maybe_plain = false; break;
-                                    case 1: maybe_auto = false; break;
-                                }
-                                break;
-                            case WR_VAR(0, 2, 2):
-                                maybe_plain = maybe_auto = maybe_second = false;
-                                break;
-                        }
-                    }
-                    break;
-                case 264: maybe_plain = maybe_auto = false; break;
-            }
-        }
-        //if (maybe_wmo)
-        //    return auto_ptr<Template>(new ShipWMO(opts, msgs));
-        if (maybe_plain)
-            return auto_ptr<Template>(new ShipPlain(opts, msgs));
-        if (maybe_auto)
-            return auto_ptr<Template>(new ShipAuto(opts, msgs));
-        if (maybe_second)
-            return auto_ptr<Template>(new ShipECMWFSecondRecord(opts, msgs));
-        // Fallback on WMO if we are confused
-        return auto_ptr<Template>(new ShipWMO(opts, msgs));
-    }
-};
-
-struct ShipPlainFactory : public TemplateFactory
-{
-    ShipPlainFactory() { name = SHIP_PLAIN_NAME; description = SHIP_PLAIN_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipPlain(opts, msgs));
-    }
-};
-struct ShipECMWFSecondRecordFactory : public TemplateFactory
-{
-    ShipECMWFSecondRecordFactory() { name = SHIP_ECMWF_SECOND_NAME; description = SHIP_ECMWF_SECOND_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipECMWFSecondRecord(opts, msgs));
-    }
-};
-struct ShipAbbrFactory : public TemplateFactory
-{
-    ShipAbbrFactory() { name = SHIP_ABBR_NAME; description = SHIP_ABBR_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipAbbr(opts, msgs));
-    }
-};
-struct ShipAutoFactory : public TemplateFactory
-{
-    ShipAutoFactory() { name = SHIP_AUTO_NAME; description = SHIP_AUTO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipAuto(opts, msgs));
-    }
-};
-struct ShipReducedFactory : public TemplateFactory
-{
-    ShipReducedFactory() { name = SHIP_REDUCED_NAME; description = SHIP_REDUCED_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipReduced(opts, msgs));
-    }
-};
-struct ShipWMOFactory : public TemplateFactory
-{
-    ShipWMOFactory() { name = SHIP_WMO_NAME; description = SHIP_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new ShipWMO(opts, msgs));
-    }
-};
 
 } // anonymous namespace
 
 void register_ship(TemplateRegistry& r)
 {
-static const TemplateFactory* ship = NULL;
-static const TemplateFactory* shipplain = NULL;
-static const TemplateFactory* shipsecond = NULL;
-static const TemplateFactory* shipabbr = NULL;
-static const TemplateFactory* shipauto = NULL;
-static const TemplateFactory* shipreduced = NULL;
-static const TemplateFactory* shipwmo = NULL;
+    r.register_factory(1, "ship", "Synop ship (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                // Scan msgs and pick the right one
+                bool maybe_wmo = true;
+                bool maybe_plain = true;
+                bool maybe_auto = true;
+                bool maybe_second = true;
+                const Msg& msg = Msg::downcast(msgs[0]);
+                for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
+                        i != msg.data.end(); ++i)
+                {
+                    const msg::Context& c = **i;
+                    switch (c.level.ltype1)
+                    {
+                        case MISSING_INT:
+                            for (std::vector<wreport::Var*>::const_iterator vi = c.data.begin();
+                                    vi != c.data.end(); ++vi)
+                            {
+                                switch ((*vi)->code())
+                                {
+                                    case WR_VAR(0, 2, 1):
+                                        switch ((*vi)->enq(0))
+                                        {
+                                            case 0: maybe_plain = false; break;
+                                            case 1: maybe_auto = false; break;
+                                        }
+                                        break;
+                                    case WR_VAR(0, 2, 2):
+                                        maybe_plain = maybe_auto = maybe_second = false;
+                                        break;
+                                }
+                            }
+                            break;
+                        case 264: maybe_plain = maybe_auto = false; break;
+                    }
+                }
+                //if (maybe_wmo)
+                //    return unique_ptr<Template>(new ShipWMO(opts, msgs));
+                if (maybe_plain)
+                    return unique_ptr<Template>(new ShipPlain(opts, msgs));
+                if (maybe_auto)
+                    return unique_ptr<Template>(new ShipAuto(opts, msgs));
+                if (maybe_second)
+                    return unique_ptr<Template>(new ShipECMWFSecondRecord(opts, msgs));
+                // Fallback on WMO if we are confused
+                return unique_ptr<Template>(new ShipWMO(opts, msgs));
+            });
 
-    if (!ship) ship = new ShipFactory;
-    if (!shipplain) shipplain = new ShipPlainFactory;
-    if (!shipsecond) shipsecond = new ShipECMWFSecondRecordFactory;
-    if (!shipabbr) shipabbr = new ShipAbbrFactory;
-    if (!shipauto) shipauto = new ShipAutoFactory;
-    if (!shipreduced) shipreduced = new ShipReducedFactory;
-    if (!shipwmo) shipwmo = new ShipWMOFactory;
+    r.register_factory(1, SHIP_PLAIN_NAME, SHIP_PLAIN_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipPlain(opts, msgs));
+            });
 
-    r.register_factory(ship);
-    r.register_factory(shipplain);
-    r.register_factory(shipsecond);
-    r.register_factory(shipabbr);
-    r.register_factory(shipauto);
-    r.register_factory(shipreduced);
-    r.register_factory(shipwmo);
+    r.register_factory(1, SHIP_ECMWF_SECOND_NAME, SHIP_ECMWF_SECOND_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipECMWFSecondRecord(opts, msgs));
+            });
+
+    r.register_factory(1, SHIP_ABBR_NAME, SHIP_ABBR_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipAbbr(opts, msgs));
+            });
+
+    r.register_factory(1, SHIP_AUTO_NAME, SHIP_AUTO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipAuto(opts, msgs));
+            });
+
+    r.register_factory(1, SHIP_REDUCED_NAME, SHIP_REDUCED_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipReduced(opts, msgs));
+            });
+
+    r.register_factory(1, SHIP_WMO_NAME, SHIP_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new ShipWMO(opts, msgs));
+            });
 }
 
 }
 }
 }
-
-/* vim:set ts=4 sw=4: */
