@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include "common.h"
 #include "msg/wr_codec.h"
 #include <wreport/bulletin.h>
-#include "msg/msgs.h"
 #include "msg/context.h"
 
 #warning TODO: remove when done
@@ -29,12 +28,6 @@
 using namespace wreport;
 using namespace std;
 
-#define SYNOP_NAME "synop"
-#define SYNOP_DESC "Synop (autodetect)"
-
-#define SYNOP_ECMWF_NAME "synop-ecmwf"
-#define SYNOP_ECMWF_DESC "Synop ECMWF (autodetect) (0.1)"
-
 #define SYNOP_WMO_NAME "synop-wmo"
 #define SYNOP_WMO_DESC "Synop WMO (0.1)"
 
@@ -59,8 +52,11 @@ struct Synop : public Template
     const msg::Context* c_sunshine1;
     const msg::Context* c_sunshine2;
     const msg::Context* c_evapo;
+    const msg::Context* c_radiation1;
+    const msg::Context* c_radiation24;
+    const msg::Context* c_tchange;
 
-    Synop(const Exporter::Options& opts, const Msgs& msgs)
+    Synop(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     void scan_levels()
@@ -68,6 +64,9 @@ struct Synop : public Template
         c_sunshine1 = NULL;
         c_sunshine2 = NULL;
         c_evapo = NULL;
+        c_radiation1 = NULL;
+        c_radiation24 = NULL;
+        c_tchange = NULL;
 
         // Scan message finding context for the data that follow
         for (std::vector<msg::Context*>::const_iterator i = msg->data.begin();
@@ -81,18 +80,41 @@ struct Synop : public Template
                     switch (c->trange.pind)
                     {
                         case 1:
-                            // msg->set(var, WR_VAR(0, 14, 31), Level(1), Trange(1, 0, time_period));
-                            if (c->find(WR_VAR(0, 14, 31)))
+                            for (const auto v: c->data)
                             {
-                                if (!c_sunshine1)
-                                    c_sunshine1 = c;
-                                else if (!c_sunshine2)
-                                    c_sunshine2 = c;
+                                switch (v->code())
+                                {
+                                    case WR_VAR(0, 14, 31):
+                                        if (!c_sunshine1)
+                                            c_sunshine1 = c;
+                                        else if (!c_sunshine2)
+                                            c_sunshine2 = c;
+                                        break;
+                                    case WR_VAR(0, 13, 33):
+                                        c_evapo = c;
+                                        break;
+                                    case WR_VAR(0, 14,  2):
+                                    case WR_VAR(0, 14,  4):
+                                    case WR_VAR(0, 14, 16):
+                                    case WR_VAR(0, 14, 28):
+                                    case WR_VAR(0, 14, 29):
+                                    case WR_VAR(0, 14, 30):
+                                        switch (c->trange.p2)
+                                        {
+                                            case  1 * 3600:
+                                                c_radiation1 = c;
+                                                break;
+                                            case 24 * 3600:
+                                                c_radiation24 = c;
+                                                break;
+                                        }
+                                        break;
+                                }
                             }
-
-                            // msg->set(var, WR_VAR(0, 13, 33), Level(1), Trange(1, 0, -time_period));
-                            if (c->find(WR_VAR(0, 13, 33)))
-                                c_evapo = c;
+                            break;
+                        case 4:
+                            if (c->find(WR_VAR(0, 12, 49)))
+                                c_tchange = c;
                             break;
                     }
                     break;
@@ -103,7 +125,7 @@ struct Synop : public Template
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
     {
         Template::to_subset(msg, subset);
-        synop.init(subset);
+        synop.init(msg, subset);
         scan_levels();
     }
 };
@@ -114,12 +136,9 @@ struct SynopECMWF : public Synop
     bool is_crex;
     Varcode prec_code;
 
-    SynopECMWF(const Exporter::Options& opts, const Msgs& msgs)
+    SynopECMWF(const Exporter::Options& opts, const Messages& msgs)
         : Synop(opts, msgs) {}
 
-    virtual const char* name() const { return SYNOP_ECMWF_NAME; }
-    virtual const char* description() const { return SYNOP_ECMWF_DESC; }
-
     void add_prec()
     {
         const Var* var = NULL;
@@ -144,16 +163,16 @@ struct SynopECMWF : public Synop
         // Use old table for old templates
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->master_table = 13;
+            b->master_table_version_number = 13;
         }
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
         // Use the best kind of precipitation found in the message to encode
         prec_code = 0;
-        for (Msgs::const_iterator mi = msgs.begin(); prec_code == 0 && mi != msgs.end(); ++mi)
+        for (Messages::const_iterator mi = msgs.begin(); prec_code == 0 && mi != msgs.end(); ++mi)
         {
-            const Msg& msg = **mi;
+            const Msg& msg = Msg::downcast(*mi);
             if (msg.get_tot_prec24_var() != NULL)
                 prec_code = WR_VAR(0, 13, 23);
             else if (msg.get_tot_prec12_var() != NULL)
@@ -168,23 +187,24 @@ struct SynopECMWF : public Synop
         if (prec_code == 0)
             prec_code = WR_VAR(0, 13, 23);
 
-        bulletin.type = 0;
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 1;
+        bulletin.data_category = 0;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 1;
     }
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
     {
         Synop::to_subset(msg, subset);
         synop.add_ecmwf_synop_head();
-        synop.add_year_to_minute();
-        synop.add_latlon_high();
+        do_D01011();
+        do_D01012();
+        do_D01021();
         /* 10 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
     }
 };
 
 struct SynopECMWFLand : public SynopECMWF
 {
-    SynopECMWFLand(const Exporter::Options& opts, const Msgs& msgs)
+    SynopECMWFLand(const Exporter::Options& opts, const Messages& msgs)
         : SynopECMWF(opts, msgs) {}
 
     virtual const char* name() const { return SYNOP_ECMWF_LAND_NAME; }
@@ -261,7 +281,7 @@ struct SynopECMWFLand : public SynopECMWF
 
 struct SynopECMWFLandHigh : public SynopECMWF
 {
-    SynopECMWFLandHigh(const Exporter::Options& opts, const Msgs& msgs)
+    SynopECMWFLandHigh(const Exporter::Options& opts, const Messages& msgs)
         : SynopECMWF(opts, msgs) {}
 
     virtual const char* name() const { return SYNOP_ECMWF_LAND_HIGH_NAME; }
@@ -328,7 +348,7 @@ struct SynopECMWFLandHigh : public SynopECMWF
 // Same as SynopECMWFLandHigh but just with a different local subtype
 struct SynopECMWFAuto : public SynopECMWFLand
 {
-    SynopECMWFAuto(const Exporter::Options& opts, const Msgs& msgs)
+    SynopECMWFAuto(const Exporter::Options& opts, const Messages& msgs)
         : SynopECMWFLand(opts, msgs) {}
 
     virtual const char* name() const { return SYNOP_ECMWF_AUTO_NAME; }
@@ -338,7 +358,7 @@ struct SynopECMWFAuto : public SynopECMWFLand
     {
         SynopECMWFLand::setupBulletin(bulletin);
 
-        bulletin.localsubtype = 3;
+        bulletin.data_subcategory_local = 3;
     }
 };
 
@@ -349,7 +369,7 @@ struct SynopWMO : public Synop
     // we process its subsets
     Bulletin* cur_bulletin;
 
-    SynopWMO(const Exporter::Options& opts, const Msgs& msgs)
+    SynopWMO(const Exporter::Options& opts, const Messages& msgs)
         : Synop(opts, msgs), cur_bulletin(0) {}
 
     virtual const char* name() const { return SYNOP_WMO_NAME; }
@@ -361,9 +381,9 @@ struct SynopWMO : public Synop
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
-        bulletin.type = 0;
-        bulletin.subtype = 255;
-        bulletin.localsubtype = 255;
+        bulletin.data_category = 0;
+        bulletin.data_subcategory = 255;
+        bulletin.data_subcategory_local = 255;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -373,6 +393,20 @@ struct SynopWMO : public Synop
         cur_bulletin = &bulletin;
     }
 
+    // SYNOP Fixed surface station identification, time, horizontal and
+    // vertical coordinates
+    void do_D01090()
+    {
+        add(WR_VAR(0,  1,  1), c_station);
+        add(WR_VAR(0,  1,  2), c_station);
+        do_station_name(WR_VAR(0,  1, 15));
+        add(WR_VAR(0,  2,  1), c_station);
+        do_D01011();
+        do_D01012();
+        do_D01021();
+        do_station_height();
+    }
+
     // D02036  Clouds with bases below station level
     void do_D02036(const Msg& msg, wreport::Subset& subset)
     {
@@ -455,7 +489,7 @@ struct SynopWMO : public Synop
     {
         add(WR_VAR(0, 20,  62), DBA_MSG_STATE_GROUND);
         add(WR_VAR(0, 13,  13), DBA_MSG_TOT_SNOW);
-        if (const Var* var = msg.find(WR_VAR(0, 12, 121), Level(1), Trange(3, 0, 43200)))
+        if (const Var* var = msg.get(WR_VAR(0, 12, 121), Level(1), Trange(3, 0, 43200)))
             subset.store_variable(WR_VAR(0, 12, 113), *var);
         else
             subset.store_variable_undef(WR_VAR(0, 12, 113));
@@ -507,20 +541,20 @@ struct SynopWMO : public Synop
 
         // Set subtype based on hour. If we have heterogeneous subsets, keep
         // the lowest of the constraints
-        int hour = synop.get_hour();
+        int hour = msg.get_datetime().hour;
         if ((hour % 6) == 0)
             // 002 at main synoptic times 00, 06, 12, 18 UTC,
-            cur_bulletin->subtype = cur_bulletin->subtype == 255 ? 2 : cur_bulletin->subtype;
+            cur_bulletin->data_subcategory = cur_bulletin->data_subcategory == 255 ? 2 : cur_bulletin->data_subcategory;
         else if ((hour % 3 == 0))
             // 001 at intermediate synoptic times 03, 09, 15, 21 UTC,
-            cur_bulletin->subtype = cur_bulletin->subtype != 0 ? 1 : cur_bulletin->subtype;
+            cur_bulletin->data_subcategory = cur_bulletin->data_subcategory != 0 ? 1 : cur_bulletin->data_subcategory;
         else
             // 000 at observation times 01, 02, 04, 05, 07, 08, 10, 11, 13, 14, 16, 17, 19, 20, 22 and 23 UTC.
-            cur_bulletin->subtype = 0;
+            cur_bulletin->data_subcategory = 0;
 
         // Fixed surface station identification, time, horizontal and vertical
         // coordinates
-        synop.add_D01090();
+        do_D01090();
 
         // D02031  Pressure data
         synop.add_D02031();
@@ -554,106 +588,67 @@ struct SynopWMO : public Synop
         }
         add(WR_VAR(0, 13, 33), c_evapo);
 
-#warning TODO
         // D02045  Radiation data (1 hour period)
-        subset.store_variable_undef(WR_VAR(0,  4, 24)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14,  2)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14,  4)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 16)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 28)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 29)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 30)); // TODO
+        if (c_radiation1)
+        {
+            subset.store_variable_d(WR_VAR(0,  4, 24), -c_radiation1->trange.p2/3600);
+            add(WR_VAR(0, 14,  2), c_radiation1);
+            add(WR_VAR(0, 14,  4), c_radiation1);
+            add(WR_VAR(0, 14, 16), c_radiation1);
+            add(WR_VAR(0, 14, 28), c_radiation1);
+            add(WR_VAR(0, 14, 29), c_radiation1);
+            add(WR_VAR(0, 14, 30), c_radiation1);
+        } else {
+            subset.store_variable_undef(WR_VAR(0,  4, 24));
+            subset.store_variable_undef(WR_VAR(0, 14,  2));
+            subset.store_variable_undef(WR_VAR(0, 14,  4));
+            subset.store_variable_undef(WR_VAR(0, 14, 16));
+            subset.store_variable_undef(WR_VAR(0, 14, 28));
+            subset.store_variable_undef(WR_VAR(0, 14, 29));
+            subset.store_variable_undef(WR_VAR(0, 14, 30));
+        }
         // D02045  Radiation data (24 hour period)
-        subset.store_variable_undef(WR_VAR(0,  4, 24)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14,  2)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14,  4)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 16)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 28)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 29)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 14, 30)); // TODO
+        if (c_radiation24)
+        {
+            subset.store_variable_d(WR_VAR(0,  4, 24), -c_radiation24->trange.p2/3600);
+            add(WR_VAR(0, 14,  2), c_radiation24);
+            add(WR_VAR(0, 14,  4), c_radiation24);
+            add(WR_VAR(0, 14, 16), c_radiation24);
+            add(WR_VAR(0, 14, 28), c_radiation24);
+            add(WR_VAR(0, 14, 29), c_radiation24);
+            add(WR_VAR(0, 14, 30), c_radiation24);
+        } else {
+            subset.store_variable_undef(WR_VAR(0,  4, 24));
+            subset.store_variable_undef(WR_VAR(0, 14,  2));
+            subset.store_variable_undef(WR_VAR(0, 14,  4));
+            subset.store_variable_undef(WR_VAR(0, 14, 16));
+            subset.store_variable_undef(WR_VAR(0, 14, 28));
+            subset.store_variable_undef(WR_VAR(0, 14, 29));
+            subset.store_variable_undef(WR_VAR(0, 14, 30));
+        }
         // D02046  Temperature change
-        subset.store_variable_undef(WR_VAR(0,  4, 24)); // TODO
-        subset.store_variable_undef(WR_VAR(0,  4, 24)); // TODO
-        subset.store_variable_undef(WR_VAR(0, 12, 49)); // TODO
-    }
-};
+        if (c_tchange)
+        {
+            // Duration of statistical processing
+            if (c_tchange->trange.p2 != MISSING_INT)
+                subset.store_variable_d(WR_VAR(0,  4, 24), -c_tchange->trange.p2 / 3600);
+            else
+                subset.store_variable_undef(WR_VAR(0,  4, 24));
 
+            // Offset from end of interval to synop reference time
+            if (c_tchange->trange.p1 != 0 && c_tchange->trange.p1 != MISSING_INT)
+                subset.store_variable_d(WR_VAR(0,  4, 24), c_tchange->trange.p1 / 3600);
+            else if (c_tchange->trange.p1 == MISSING_INT || c_tchange->trange.p2 == MISSING_INT)
+                subset.store_variable_undef(WR_VAR(0,  4, 24));
+            else
+                subset.store_variable_d(WR_VAR(0,  4, 24), 0);
 
-struct SynopWMOFactory : public virtual TemplateFactory
-{
-    SynopWMOFactory() { name = SYNOP_WMO_NAME; description = SYNOP_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        // Scan msgs and pick the right one
-        return auto_ptr<Template>(new SynopWMO(opts, msgs));
-    }
-};
-
-struct SynopECMWFLandFactory : public virtual TemplateFactory
-{
-    SynopECMWFLandFactory() { name = SYNOP_ECMWF_LAND_NAME; description = SYNOP_ECMWF_LAND_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new SynopECMWFLand(opts, msgs));
-    }
-};
-
-struct SynopECMWFLandHighFactory : public virtual TemplateFactory
-{
-    SynopECMWFLandHighFactory() { name = SYNOP_ECMWF_LAND_HIGH_NAME; description = SYNOP_ECMWF_LAND_HIGH_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
-    }
-};
-
-struct SynopECMWFAutoFactory : public virtual TemplateFactory
-{
-    SynopECMWFAutoFactory() { name = SYNOP_ECMWF_AUTO_NAME; description = SYNOP_ECMWF_AUTO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new SynopECMWFAuto(opts, msgs));
-    }
-};
-
-struct SynopECMWFFactory : public virtual TemplateFactory
-{
-    SynopECMWFFactory() { name = SYNOP_ECMWF_NAME; description = SYNOP_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        const Msg& msg = *msgs[0];
-        const Var* var = msg.get_st_type_var();
-        if (var != NULL && var->enqi() == 0)
-            return auto_ptr<Template>(new SynopECMWFAuto(opts, msgs));
-
-        // If it has a geopotential, it's a land high station
-        for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
-                i != msg.data.end(); ++i)
-            if ((*i)->level.ltype1 == 100)
-                if (const Var* v = (*i)->find(WR_VAR(0, 10, 8)))
-                    return auto_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
-
-        return auto_ptr<Template>(new SynopECMWFLand(opts, msgs));
-    }
-};
-
-struct SynopFactory : public SynopECMWFFactory, SynopWMOFactory
-{
-    SynopFactory() { name = SYNOP_NAME; description = SYNOP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        const Msg& msg = *msgs[0];
-        const Var* var = msg.get_st_name_var();
-        if (var)
-            return SynopWMOFactory::make(opts, msgs);
-        else
-            return SynopECMWFFactory::make(opts, msgs);
+            add(WR_VAR(0, 12, 49), c_tchange);
+        } else {
+            subset.store_variable_undef(WR_VAR(0,  4, 24));
+            subset.store_variable_undef(WR_VAR(0,  4, 24));
+            subset.store_variable_undef(WR_VAR(0, 12, 49));
+        }
     }
 };
 
@@ -662,29 +657,55 @@ struct SynopFactory : public SynopECMWFFactory, SynopWMOFactory
 
 void register_synop(TemplateRegistry& r)
 {
-static const TemplateFactory* synop = NULL;
-static const TemplateFactory* synopwmo = NULL;
-static const TemplateFactory* synopecmwf = NULL;
-static const TemplateFactory* synopecmwfland = NULL;
-static const TemplateFactory* synopecmwflandhigh = NULL;
-static const TemplateFactory* synopecmwfauto = NULL;
+    r.register_factory(0, "synop", "Synop (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                const Msg& msg = Msg::downcast(msgs[0]);
+                const Var* var = msg.get_st_name_var();
+                if (var)
+                {
+                    const wr::TemplateFactory& fac = wr::TemplateRegistry::get(SYNOP_WMO_NAME);
+                    return fac.factory(opts, msgs);
+                }
+                else
+                {
+                    const wr::TemplateFactory& fac = wr::TemplateRegistry::get("synop-ecmwf");
+                    return fac.factory(opts, msgs);
+                }
+            });
+    r.register_factory(0, SYNOP_WMO_NAME, SYNOP_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new SynopWMO(opts, msgs));
+            });
+    r.register_factory(0, "synop-ecmwf", "Synop ECMWF (autodetect) (0.1)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                const Msg& msg = Msg::downcast(msgs[0]);
+                const Var* var = msg.get_st_type_var();
+                if (var != NULL && var->enqi() == 0)
+                    return unique_ptr<Template>(new SynopECMWFAuto(opts, msgs));
 
-    if (!synop) synop = new SynopFactory;
-    if (!synopwmo) synopwmo = new SynopWMOFactory;
-    if (!synopecmwf) synopecmwf = new SynopECMWFFactory;
-    if (!synopecmwfland) synopecmwfland = new SynopECMWFLandFactory;
-    if (!synopecmwflandhigh) synopecmwflandhigh = new SynopECMWFLandHighFactory;
-    if (!synopecmwfauto) synopecmwfauto = new SynopECMWFAutoFactory;
+                // If it has a geopotential, it's a land high station
+                for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
+                        i != msg.data.end(); ++i)
+                    if ((*i)->level.ltype1 == 100)
+                        if (const Var* v = (*i)->find(WR_VAR(0, 10, 8)))
+                            return unique_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
 
-    r.register_factory(synop);
-    r.register_factory(synopwmo);
-    r.register_factory(synopecmwf);
-    r.register_factory(synopecmwfland);
-    r.register_factory(synopecmwflandhigh);
-    r.register_factory(synopecmwfauto);
+                return unique_ptr<Template>(new SynopECMWFLand(opts, msgs));
+            });
+    r.register_factory(0, SYNOP_ECMWF_LAND_NAME, SYNOP_ECMWF_LAND_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new SynopECMWFLand(opts, msgs));
+            });
+    r.register_factory(0, SYNOP_ECMWF_LAND_HIGH_NAME, SYNOP_ECMWF_LAND_HIGH_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
+            });
+    r.register_factory(0, SYNOP_ECMWF_AUTO_NAME, SYNOP_ECMWF_AUTO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new SynopECMWFAuto(opts, msgs));
+            });
 }
 
 }
 }
 }
-/* vim:set ts=4 sw=4: */
