@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2012  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,28 +18,20 @@
  */
 
 #include "msg/wr_codec.h"
+#include "msg/msg.h"
+#include "msg/context.h"
 #include <wreport/bulletin.h>
+#include <wreport/vartable.h>
 #include <wreport/conv.h>
 #include <wreport/codetables.h>
 #include <wreport/notes.h>
-#include "msg/msgs.h"
-#include "msg/context.h"
 
 using namespace wreport;
 using namespace std;
 
-#define TEMP_NAME "temp"
-#define TEMP_DESC "Temp (autodetect)"
-
-#define TEMP_SHIP_NAME "temp-ship"
-#define TEMP_SHIP_DESC "Temp ship (autodetect)"
-
 #define TEMP_WMO_NAME "temp-wmo"
 #define TEMP_WMO_DESC "Temp WMO (2.101)"
 
-#define TEMP_ECMWF_NAME "temp-ecmwf"
-#define TEMP_ECMWF_DESC "Temp ECMWF (autodetect)"
-
 #define TEMP_ECMWF_LAND_NAME "temp-ecmwf-land"
 #define TEMP_ECMWF_LAND_DESC "Temp ECMWF land (2.101)"
 
@@ -49,9 +41,6 @@ using namespace std;
 #define TEMP_RADAR_NAME "temp-radar"
 #define TEMP_RADAR_DESC "Temp radar doppler wind profile (6.1)"
 
-#define PILOT_NAME "pilot"
-#define PILOT_DESC "pilot (autodetect)"
-
 #define PILOT_WMO_NAME "pilot-wmo"
 #define PILOT_WMO_DESC "Pilot (2.1, 2.2, 2.3)"
 
@@ -70,7 +59,7 @@ struct TempBase : public Template
 {
     bool is_crex;
 
-    TempBase(const Exporter::Options& opts, const Msgs& msgs)
+    TempBase(const Exporter::Options& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     /// Count the number of sounding levels
@@ -122,8 +111,8 @@ struct TempBase : public Template
                     // Deal with 'missing VSS' group markers
                     subset->store_variable_undef(WR_VAR(0, 8, 1));
                 } else {
-                    Var nvar(subset->btable->query(WR_VAR(0, 8, 1)), vssval);
-                    nvar.copy_attrs(*vss);
+                    Var nvar(subset->tables->btable->query(WR_VAR(0, 8, 1)), vssval);
+                    nvar.setattrs(*vss);
                     subset->store_variable(WR_VAR(0, 8, 1), nvar);
                 }
             }
@@ -146,25 +135,25 @@ struct TempBase : public Template
         // Use old table for old templates
         if (BufrBulletin* b = dynamic_cast<BufrBulletin*>(&bulletin))
         {
-            b->master_table = 13;
+            b->master_table_version_number = 13;
         }
 
         is_crex = dynamic_cast<CrexBulletin*>(&bulletin) != 0;
 
-        bulletin.type = 2;
-        bulletin.subtype = 255;
+        bulletin.data_category = 2;
+        bulletin.data_subcategory = 255;
 
         // Scan to see what we are dealing with
-        bulletin.localsubtype = 101;
-        for (Msgs::const_iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+        bulletin.data_subcategory_local = 101;
+        for (const auto& mi: msgs)
         {
-            const Msg& msg = **mi;
+            const Msg& msg = Msg::downcast(mi);
             if (msg.type == MSG_PILOT)
             {
-                bulletin.localsubtype = 91;
+                bulletin.data_subcategory_local = 91;
                 break;
             } else if (msg.get_ident_var()) {
-                bulletin.localsubtype = 102;
+                bulletin.data_subcategory_local = 102;
                 break;
             } else if (msg.get_block_var()) {
                 break;
@@ -210,7 +199,7 @@ struct TempBase : public Template
 
 struct TempWMO : public TempBase
 {
-    TempWMO(const Exporter::Options& opts, const Msgs& msgs)
+    TempWMO(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_WMO_NAME; }
@@ -220,15 +209,15 @@ struct TempWMO : public TempBase
     {
         TempBase::setupBulletin(bulletin);
 
-        bulletin.type = 2;
-        bulletin.subtype = 4;
-        bulletin.localsubtype = 255;
-        for (Msgs::const_iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+        bulletin.data_category = 2;
+        bulletin.data_subcategory = 4;
+        bulletin.data_subcategory_local = 255;
+        for (const auto& mi : msgs)
         {
-            const Msg& msg = **mi;
+            const Msg& msg = Msg::downcast(mi);
             if (msg.type == MSG_TEMP_SHIP)
             {
-                bulletin.subtype = 5;
+                bulletin.data_subcategory = 5;
                 break;
             }
         }
@@ -365,7 +354,7 @@ struct TempWMO : public TempBase
 
 struct TempRadar : public TempBase
 {
-    TempRadar(const Exporter::Options& opts, const Msgs& msgs)
+    TempRadar(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_RADAR_NAME; }
@@ -375,9 +364,9 @@ struct TempRadar : public TempBase
     {
         TempBase::setupBulletin(bulletin);
 
-        bulletin.type = 6;
-        bulletin.subtype = 1;
-        bulletin.localsubtype = 255;
+        bulletin.data_category = 6;
+        bulletin.data_subcategory = 1;
+        bulletin.data_subcategory_local = 255;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -441,7 +430,7 @@ struct TempRadar : public TempBase
                 if (const wreport::Var *var = c->find(code))
                 {
                     add(code, var);
-                    add(code, var->enqa(WR_VAR(0, 33,  2)));
+                    add(WR_VAR(0, 33, 2), var->enqa(WR_VAR(0, 33,  2)));
                 } else {
                     subset.store_variable_undef(code);
                     subset.store_variable_undef(WR_VAR(0, 33,   2));
@@ -456,7 +445,7 @@ struct TempRadar : public TempBase
 
 struct TempEcmwfLand : public TempBase
 {
-    TempEcmwfLand(const Exporter::Options& opts, const Msgs& msgs)
+    TempEcmwfLand(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_ECMWF_LAND_NAME; }
@@ -492,11 +481,8 @@ struct TempEcmwfLand : public TempBase
         /*  1 */ add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
         /*  2 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
         /*  3 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
-        /*  4 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /*  5 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /*  6 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /*  7 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /*  8 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        do_D01011();
+        do_D01012();
         /*  9 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
         /* 10 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
         /* 11 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
@@ -526,7 +512,7 @@ struct TempEcmwfLand : public TempBase
 
 struct TempEcmwfShip : public TempBase
 {
-    TempEcmwfShip(const Exporter::Options& opts, const Msgs& msgs)
+    TempEcmwfShip(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_ECMWF_SHIP_NAME; }
@@ -578,11 +564,8 @@ struct TempEcmwfShip : public TempBase
         /*  2 */ add(WR_VAR(0,  1, 13), DBA_MSG_ST_SPEED);
         /*  3 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
         /*  4 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
-        /*  5 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /*  6 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /*  7 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /*  8 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /*  9 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        do_D01011();
+        do_D01012();
         /* 10 */ add(WR_VAR(0,  5,  2), DBA_MSG_LATITUDE);
         /* 11 */ add(WR_VAR(0,  6,  2), DBA_MSG_LONGITUDE);
         /* 12 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
@@ -614,7 +597,7 @@ struct PilotWMO : public TempBase
 {
     bool pressure_levs;
 
-    PilotWMO(const Exporter::Options& opts, const Msgs& msgs)
+    PilotWMO(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return PILOT_WMO_NAME; }
@@ -626,9 +609,9 @@ struct PilotWMO : public TempBase
         unsigned has_press = 0;
         unsigned has_height = 0;
 
-        for (Msgs::const_iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+        for (const auto& mi: msgs)
         {
-            const Msg& msg = **mi;
+            const Msg& msg = Msg::downcast(mi);
             for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
                     i != msg.data.end(); ++i)
             {
@@ -651,9 +634,9 @@ struct PilotWMO : public TempBase
     {
         TempBase::setupBulletin(bulletin);
 
-        bulletin.type = 2;
-        bulletin.subtype = 1;
-        bulletin.localsubtype = 255;
+        bulletin.data_category = 2;
+        bulletin.data_subcategory = 1;
+        bulletin.data_subcategory_local = 255;
 
         // Data descriptor section
         bulletin.datadesc.clear();
@@ -677,12 +660,8 @@ struct PilotWMO : public TempBase
 
         // Date/time of launch
         add(WR_VAR(0,  8, 21), 18); // Launch time
-        add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
-        add(WR_VAR(0,  4,  6), DBA_MSG_SECOND);
+        do_D01011();
+        do_D01013();
 
         // Horizontal and vertical coordinates of launch site
         add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
@@ -760,7 +739,7 @@ struct PilotWMO : public TempBase
 
 struct PilotEcmwf : public TempBase
 {
-    PilotEcmwf(const Exporter::Options& opts, const Msgs& msgs)
+    PilotEcmwf(const Exporter::Options& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return PILOT_ECMWF_NAME; }
@@ -808,11 +787,8 @@ struct PilotEcmwf : public TempBase
         /*  1 */ add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
         /*  2 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
         /*  3 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
-        /*  4 */ add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
-        /*  5 */ add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
-        /*  6 */ add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
-        /*  7 */ add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
-        /*  8 */ add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        do_D01011();
+        do_D01012();
         /*  9 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
         /* 10 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
         /* 11 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
@@ -839,8 +815,8 @@ struct PilotEcmwf : public TempBase
 
             /* Add vertical sounding significance */
             {
-                Var nvar(subset.btable->query(WR_VAR(0, 8, 1)), convert_BUFR08042_to_BUFR08001(vss->enqi()));
-                nvar.copy_attrs(*vss);
+                Var nvar(subset.tables->btable->query(WR_VAR(0, 8, 1)), (int)convert_BUFR08042_to_BUFR08001(vss->enqi()));
+                nvar.setattrs(*vss);
                 subset.store_variable(WR_VAR(0, 8, 1), nvar);
             }
 
@@ -888,162 +864,80 @@ struct PilotEcmwf : public TempBase
 };
 
 
-struct TempWMOFactory : public virtual TemplateFactory
-{
-    TempWMOFactory() { name = TEMP_WMO_NAME; description = TEMP_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new TempWMO(opts, msgs));
-    }
-};
-struct TempEcmwfLandFactory : public virtual TemplateFactory
-{
-    TempEcmwfLandFactory() { name = TEMP_ECMWF_LAND_NAME; description = TEMP_ECMWF_LAND_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new TempEcmwfLand(opts, msgs));
-    }
-};
-struct TempEcmwfShipFactory : public virtual TemplateFactory
-{
-    TempEcmwfShipFactory() { name = TEMP_ECMWF_SHIP_NAME; description = TEMP_ECMWF_SHIP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new TempEcmwfShip(opts, msgs));
-    }
-};
-struct TempEcmwfFactory : public virtual TemplateFactory
-{
-    TempEcmwfFactory() { name = TEMP_ECMWF_NAME; description = TEMP_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        if (msgs.empty() || msgs[0]->type != MSG_TEMP_SHIP)
-            return auto_ptr<Template>(new TempEcmwfLand(opts, msgs));
-        else
-            return auto_ptr<Template>(new TempEcmwfShip(opts, msgs));
-    }
-};
-struct TempShipFactory : public virtual TemplateFactory
-{
-    TempShipFactory() { name = TEMP_SHIP_NAME; description = TEMP_SHIP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new TempEcmwfShip(opts, msgs));
-    }
-};
-struct TempRadarFactory : public virtual TemplateFactory
-{
-    TempRadarFactory() { name = TEMP_RADAR_NAME; description = TEMP_RADAR_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new TempRadar(opts, msgs));
-    }
-};
-struct TempFactory : public TempEcmwfFactory, public TempWMOFactory, public TempRadarFactory
-{
-    TempFactory() { name = TEMP_NAME; description = TEMP_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        const Msg& msg = *msgs[0];
-
-        // Get the type of equipment used
-        if (const wreport::Var* var = msg.find(WR_VAR(0, 2, 3), Level(1), Trange::instant()))
-            // Is it a Radar?
-            if (var->enq(0) == 3)
-                return TempRadarFactory::make(opts, msgs);
-
-        // ECMWF temps use normal replication which cannot do more than 256 levels
-        if (msg.data.size() > 260)
-            return TempWMOFactory::make(opts, msgs);
-        const Var* var = msg.get_sonde_tracking_var();
-        if (var)
-            return TempWMOFactory::make(opts, msgs);
-        else
-            return TempEcmwfFactory::make(opts, msgs);
-    }
-};
-struct PilotWMOFactory : public virtual TemplateFactory
-{
-    PilotWMOFactory() { name = PILOT_WMO_NAME; description = PILOT_WMO_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new PilotWMO(opts, msgs));
-    }
-};
-struct PilotEcmwfFactory : public virtual TemplateFactory
-{
-    PilotEcmwfFactory() { name = PILOT_ECMWF_NAME; description = PILOT_ECMWF_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        return auto_ptr<Template>(new PilotEcmwf(opts, msgs));
-    }
-};
-struct PilotFactory : public virtual PilotEcmwfFactory, PilotWMOFactory
-{
-    PilotFactory() { name = PILOT_NAME; description = PILOT_DESC; }
-
-    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
-    {
-        const Msg& msg = *msgs[0];
-        const Var* var = msg.get_sonde_tracking_var();
-        // Try with another one in case the first was just unset
-        if (!var) var = msg.get_meas_equip_type_var();
-        if (var)
-            return PilotWMOFactory::make(opts, msgs);
-        else
-            return PilotEcmwfFactory::make(opts, msgs);
-    }
-};
-
 } // anonymous namespace
 
 void register_temp(TemplateRegistry& r)
 {
-static const TemplateFactory* temp = NULL;
-static const TemplateFactory* tempship = NULL;
-static const TemplateFactory* tempwmo = NULL;
-static const TemplateFactory* tempecmwf = NULL;
-static const TemplateFactory* tempecmwfland = NULL;
-static const TemplateFactory* tempecmwfship = NULL;
-static const TemplateFactory* tempradar = NULL;
-static const TemplateFactory* pilot = NULL;
-static const TemplateFactory* pilotwmo = NULL;
-static const TemplateFactory* pilotecmwf = NULL;
+    r.register_factory(2, "temp", "Temp (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                const Msg& msg = Msg::downcast(msgs[0]);
 
-    if (!temp) temp = new TempFactory;
-    if (!tempship) tempship = new TempShipFactory;
-    if (!tempwmo) tempwmo = new TempWMOFactory;
-    if (!tempecmwf) tempecmwf = new TempEcmwfFactory;
-    if (!tempecmwfland) tempecmwfland = new TempEcmwfLandFactory;
-    if (!tempecmwfship) tempecmwfship = new TempEcmwfShipFactory;
-    if (!tempradar) tempradar = new TempRadarFactory;
-    if (!pilot) pilot = new PilotFactory;
-    if (!pilotwmo) pilotwmo = new PilotWMOFactory;
-    if (!pilotecmwf) pilotecmwf = new PilotEcmwfFactory;
+                // Get the type of equipment used
+                if (const wreport::Var* var = msg.get(WR_VAR(0, 2, 3), Level(1), Trange::instant()))
+                    // Is it a Radar?
+                    if (var->enq(0) == 3)
+                        return unique_ptr<Template>(new TempRadar(opts, msgs));
 
-    r.register_factory(temp);
-    r.register_factory(tempship);
-    r.register_factory(tempwmo);
-    r.register_factory(tempecmwf);
-    r.register_factory(tempecmwfland);
-    r.register_factory(tempecmwfship);
-    r.register_factory(tempradar);
-    r.register_factory(pilot);
-    r.register_factory(pilotwmo);
-    r.register_factory(pilotecmwf);
+                // ECMWF temps use normal replication which cannot do more than 256 levels
+                if (msg.data.size() > 260)
+                    return unique_ptr<Template>(new TempWMO(opts, msgs));
+                const Var* var = msg.get_sonde_tracking_var();
+                if (var)
+                    return unique_ptr<Template>(new TempWMO(opts, msgs));
+                else
+                {
+                    const wr::TemplateFactory& fac = wr::TemplateRegistry::get("temp-ecmwf");
+                    return fac.factory(opts, msgs);
+                }
+            });
+    r.register_factory(2, "temp-ship", "Temp ship (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
+            });
+    r.register_factory(2, TEMP_WMO_NAME, TEMP_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new TempWMO(opts, msgs));
+            });
+    r.register_factory(2, "temp-ecmwf", "Temp ECMWF (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                if (msgs.empty() || Msg::downcast(msgs[0]).type != MSG_TEMP_SHIP)
+                    return unique_ptr<Template>(new TempEcmwfLand(opts, msgs));
+                else
+                    return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
+            });
+    r.register_factory(2, TEMP_ECMWF_LAND_NAME, TEMP_ECMWF_LAND_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new TempEcmwfLand(opts, msgs));
+            });
+    r.register_factory(2, TEMP_ECMWF_SHIP_NAME, TEMP_ECMWF_SHIP_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
+            });
+    r.register_factory(6, TEMP_RADAR_NAME, TEMP_RADAR_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new TempRadar(opts, msgs));
+            });
+    r.register_factory(2, PILOT_WMO_NAME, PILOT_WMO_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new PilotWMO(opts, msgs));
+            });
+    r.register_factory(2, PILOT_ECMWF_NAME, PILOT_ECMWF_DESC,
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                return unique_ptr<Template>(new PilotEcmwf(opts, msgs));
+            });
+    r.register_factory(2, "pilot", "pilot (autodetect)",
+            [](const Exporter::Options& opts, const Messages& msgs) {
+                const Msg& msg = Msg::downcast(msgs[0]);
+                const Var* var = msg.get_sonde_tracking_var();
+                // Try with another one in case the first was just unset
+                if (!var) var = msg.get_meas_equip_type_var();
+                if (var)
+                    return unique_ptr<Template>(new PilotWMO(opts, msgs));
+                else
+                    return unique_ptr<Template>(new PilotEcmwf(opts, msgs));
+            });
 }
 
 }
 }
 }
-
-/* vim:set ts=4 sw=4: */
