@@ -1,7 +1,7 @@
 /*
  * dballe/wr_importers/base - Base infrastructure for wreport importers
  *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  */
 
 #include "base.h"
-#include "msg/msgs.h"
+#include "core/var.h"
 #include <wreport/bulletin.h>
 #include <cstdlib>
 #include <iostream>
@@ -43,8 +43,39 @@ static const Level lev_std_wind(103, 10*1000);
 static const Trange tr_std_wind(200, 0, 600);
 static const Trange tr_std_wind_max10m(205, 0, 600);
 
+void Importer::init()
+{
+    ye = mo = da = ho = mi = se = MISSING_INT;
+}
 
-std::auto_ptr<Importer> Importer::createSat(const msg::Importer::Options&) { throw error_unimplemented("WB sat Importers"); }
+void Importer::import(const wreport::Subset& subset, Msg& msg)
+{
+    this->subset = &subset;
+    this->msg = &msg;
+    init();
+    run();
+    if (ye == MISSING_INT)
+        msg.set_datetime(Datetime());
+    else
+    {
+        if (mo == MISSING_INT)
+            throw error_consistency("no month information found in message to import");
+        if (da == MISSING_INT)
+            throw error_consistency("no day information found in message to import");
+        if (ho == MISSING_INT)
+            throw error_consistency("no hour information found in message to import");
+        if (mi == MISSING_INT)
+            throw error_consistency("no minute information found in message to import");
+        if (se == MISSING_INT)
+            se = 0;
+        // Accept an hour of 24:00:00 and move it to 00:00:00 of the following
+        // day
+        Datetime::normalise_h24(ye, mo, da, ho, mi, se);
+        msg.set_datetime(Datetime(ye, mo, da, ho, mi, se));
+    }
+}
+
+std::unique_ptr<Importer> Importer::createSat(const msg::Importer::Options&) { throw error_unimplemented("WB sat Importers"); }
 
 void WMOImporter::import_var(const Var& var)
 {
@@ -61,12 +92,12 @@ void WMOImporter::import_var(const Var& var)
 		case WR_VAR(0,  1, 63): msg->set_st_name_icao_var(var); break;
 		case WR_VAR(0,  2,  1): msg->set_st_type_var(var); break;
 		case WR_VAR(0,  1, 15): msg->set_st_name_var(var); break;
-		case WR_VAR(0,  4,  1): msg->set_year_var(var); break;
-		case WR_VAR(0,  4,  2): msg->set_month_var(var); break;
-		case WR_VAR(0,  4,  3): msg->set_day_var(var); break;
-		case WR_VAR(0,  4,  4): msg->set_hour_var(var); break;
-		case WR_VAR(0,  4,  5): msg->set_minute_var(var); break;
-		case WR_VAR(0,  4,  6): msg->set_second_var(var); break;
+        case WR_VAR(0,  4,  1): ye = var.enqi(); if (var.next_attr()) msg->set_year_var(var); break;
+        case WR_VAR(0,  4,  2): mo = var.enqi(); if (var.next_attr()) msg->set_month_var(var); break;
+        case WR_VAR(0,  4,  3): da = var.enqi(); if (var.next_attr()) msg->set_day_var(var); break;
+        case WR_VAR(0,  4,  4): ho = var.enqi(); if (var.next_attr()) msg->set_hour_var(var); break;
+        case WR_VAR(0,  4,  5): mi = var.enqi(); if (var.next_attr()) msg->set_minute_var(var); break;
+        case WR_VAR(0,  4,  6): se = var.enqi(); if (var.next_attr()) msg->set_second_var(var); break;
 		case WR_VAR(0,  5,  1):
 		case WR_VAR(0,  5,  2): msg->set_latitude_var(var); break;
 		case WR_VAR(0,  6,  1):
@@ -127,12 +158,12 @@ void TimerangeContext::peek_var(const Var& var, unsigned pos)
             case WR_VAR(0,  4,  4): hour = var.enqi(); break;
             case WR_VAR(0,  4, 24):
                 // Time period in hours
-                if ((int)pos == last_B04024_pos + 1 && var.enqi() != 0)
+                if ((int)pos == last_B04024_pos + 1)
                 {
                     // Cope with the weird idea of using B04024 twice to indicate
                     // beginning and end of a period not ending with the SYNOP
                     // reference time
-                    if (time_period != MISSING_INT)
+                    if (time_period != MISSING_INT && var.enqi() != 0)
                     {
                         time_period -= var.enqd() * 3600;
                         time_period_offset = var.enqd() * 3600;
@@ -153,6 +184,13 @@ void TimerangeContext::peek_var(const Var& var, unsigned pos)
             case WR_VAR(0,  8, 21):
                 // Time significance
                 time_sig = var.enqi();
+                // If we get time significance 18 "Radiosonde launch time"
+                // before getting the initial reference time, they are trying
+                // to tell us that they are giving us the radiosonde launch
+                // time: ignore it, since it is already what we gave for
+                // granted.
+                if (hour == MISSING_INT and time_sig == 18)
+                    time_sig = MISSING_TIME_SIG;
                 break;
         }
     } else {
@@ -371,9 +409,9 @@ void ContextChooser::ib_trange_use_shorcut_if_standard_else_real(const Trange& s
 
 void ContextChooser::ib_set()
 {
-    auto_ptr<Var> handover(var);
+    unique_ptr<Var> handover(var);
     var = 0;
-    msg->set(handover, chosen_lev, chosen_tr);
+    msg->set(move(handover), chosen_lev, chosen_tr);
 }
 
 void ContextChooser::set_gen_sensor(const Var& var, Varcode code, const Level& defaultLevel, const Trange& trange)
@@ -395,7 +433,7 @@ void ContextChooser::set_gen_sensor(const Var& var, Varcode code, const Level& d
     else if (simplified)
     {
         Var var1(var);
-        var1.seta(newvar(WR_VAR(0, 7, 32), level.height_sensor));
+        var1.seta(auto_ptr<Var>(newvar(WR_VAR(0, 7, 32), level.height_sensor).release()));
         msg->set(var1, code, defaultLevel, trange);
     } else
         msg->set(var, code, Level(103, level.height_sensor * 1000), trange);
@@ -483,8 +521,9 @@ void ContextChooser::set_wind_max(const wreport::Var& var, int shortcut)
 void ContextChooser::set_pressure(const wreport::Var& var)
 {
     if (level.press_std == MISSING_PRESS_STD)
-        throw error_consistency("B10009 given without pressure of standard level");
-    msg->set(var, WR_VAR(0, 10,  8), Level(100, level.press_std), Trange::instant());
+        msg->set(var, WR_VAR(0, 10,  8), Level(100), Trange::instant());
+    else
+        msg->set(var, WR_VAR(0, 10,  8), Level(100, level.press_std), Trange::instant());
 }
 
 void ContextChooser::set_water_temperature(const wreport::Var& var)
@@ -497,7 +536,7 @@ void ContextChooser::set_water_temperature(const wreport::Var& var)
 
 void ContextChooser::set_swell_waves(const wreport::Var& var)
 {
-    msg->set(var, var.code(), Level::waves(261, level.swell_wave_group), Trange::instant());
+    msg->set(var, var.code(), Level(264, MISSING_INT, 261, level.swell_wave_group), Trange::instant());
 }
 
 SynopBaseImporter::SynopBaseImporter(const msg::Importer::Options& opts)
