@@ -1,7 +1,7 @@
 /*
  * dballe/wr_importers/base - Base infrastructure for wreport importers
  *
- * Copyright (C) 2005--2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,9 @@ protected:
     const msg::Importer::Options& opts;
     const wreport::Subset* subset;
     Msg* msg;
+    int ye, mo, da, ho, mi, se;
 
-    virtual void init() {}
+    virtual void init();
     virtual void run() = 0;
 
 public:
@@ -52,23 +53,17 @@ public:
 
     virtual MsgType scanType(const wreport::Bulletin& bulletin) const = 0;
 
-    void import(const wreport::Subset& subset, Msg& msg)
-    {
-	    this->subset = &subset;
-	    this->msg = &msg;
-	    init();
-	    run();
-    }
+    void import(const wreport::Subset& subset, Msg& msg);
 
-    static std::auto_ptr<Importer> createSynop(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createShip(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createMetar(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createTemp(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createPilot(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createFlight(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createSat(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createPollution(const msg::Importer::Options&);
-    static std::auto_ptr<Importer> createGeneric(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createSynop(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createShip(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createMetar(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createTemp(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createPilot(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createFlight(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createSat(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createPollution(const msg::Importer::Options&);
+    static std::unique_ptr<Importer> createGeneric(const msg::Importer::Options&);
 };
 
 class WMOImporter : public Importer
