@@ -18,6 +18,7 @@
  */
 
 #include "base.h"
+#include "dballe/core/var.h"
 #include <wreport/bulletin.h>
 #include <wreport/subset.h>
 #include <wreport/conv.h>
@@ -65,7 +66,7 @@ public:
         {
             // If we don't have a level yet, defer adding the variable until we
             // have one
-            auto_ptr<Var> copy(var_copy_without_unset_attrs(var));
+            unique_ptr<Var> copy(var_copy_without_unset_attrs(var));
             deferred.push_back(copy.release());
         }
         else
@@ -78,7 +79,7 @@ public:
         {
             // If we don't have a level yet, defer adding the variable until we
             // have one
-            auto_ptr<Var> copy(var_copy_without_unset_attrs(var, code));
+            unique_ptr<Var> copy(var_copy_without_unset_attrs(var, code));
             deferred.push_back(copy.release());
         }
         else
@@ -96,9 +97,9 @@ public:
         for (vector<Var*>::iterator i = deferred.begin();
                 i != deferred.end(); ++i)
         {
-            auto_ptr<Var> var(*i);
+            unique_ptr<Var> var(*i);
             *i = 0;
-            msg->set(var, lev, Trange::instant());
+            msg->set(move(var), lev, Trange::instant());
         }
         deferred.clear();
     }
@@ -109,7 +110,7 @@ public:
         {
             const Var& var = (*subset)[pos];
             if (WR_VAR_F(var.code()) != 0) continue;
-            if (var.value() != NULL)
+            if (var.isset())
                 import_var(var);
         }
         if (b01008)
@@ -138,7 +139,7 @@ public:
 
     MsgType scanType(const Bulletin& bulletin) const
     {
-        switch (bulletin.localsubtype)
+        switch (bulletin.data_subcategory_local)
         {
             case 142: return MSG_AIREP;
             case 144: return MSG_AMDAR;
@@ -152,9 +153,9 @@ public:
     }
 };
 
-std::auto_ptr<Importer> Importer::createFlight(const msg::Importer::Options& opts)
+std::unique_ptr<Importer> Importer::createFlight(const msg::Importer::Options& opts)
 {
-    return auto_ptr<Importer>(new FlightImporter(opts));
+    return unique_ptr<Importer>(new FlightImporter(opts));
 }
 
 void FlightImporter::import_var(const Var& var)
