@@ -23,6 +23,7 @@
 #include <wreport/conv.h>
 #include <wreport/codetables.h>
 #include <dballe/msg/context.h>
+#include <dballe/core/verbose.h>
 #include <cmath>
 
 // Define to debug the sounding group matching algorithm
@@ -144,7 +145,7 @@ public:
         for (pos = 0; pos < subset->size(); )
         {
             const Var& var = (*subset)[pos];
-            if (WR_VAR_F(var.code()) != 0 || var.value() == NULL)
+            if (WR_VAR_F(var.code()) != 0 || !var.isset())
             {
                 // Ignore non-B variables and variables that are unset
                 ++pos;
@@ -165,39 +166,39 @@ public:
         /* Extract surface data from the surface level */
         if (surface_press != -1)
         {
-                // Pressure is taken from a saved variable referencing to the original
-                // pressure data in the message, to preserve data attributes
-                if (surface_press_var && surface_press_var->value() && msg->get_press_var())
-                        msg->set_press_var(*surface_press_var);
+            // Pressure is taken from a saved variable referencing to the original
+            // pressure data in the message, to preserve data attributes
+            if (surface_press_var && surface_press_var->isset() && msg->get_press_var())
+                msg->set_press_var(*surface_press_var);
 
-                const Context* sfc = msg->find_context(Level(100, surface_press), Trange::instant());
-                if (sfc != NULL)
-                {
-                        const Var* var = sfc->find(WR_VAR(0, 12, 1));
-                        if (var && msg->get_temp_2m_var())
-                                msg->set_temp_2m_var(*var);
+            const Context* sfc = msg->find_context(Level(100, surface_press), Trange::instant());
+            if (sfc != NULL)
+            {
+                const Var* var = sfc->find(WR_VAR(0, 12, 1));
+                if (var && msg->get_temp_2m_var())
+                    msg->set_temp_2m_var(*var);
 
-                        var = sfc->find(WR_VAR(0, 12, 3));
-                        if (var && msg->get_dewpoint_2m_var())
-                                msg->set_dewpoint_2m_var(*var);
+                var = sfc->find(WR_VAR(0, 12, 3));
+                if (var && msg->get_dewpoint_2m_var())
+                    msg->set_dewpoint_2m_var(*var);
 
-                        var = sfc->find(WR_VAR(0, 11, 1));
-                        if (var && !msg->get_wind_dir_var())
-                                msg->set_wind_dir_var(*var);
+                var = sfc->find(WR_VAR(0, 11, 1));
+                if (var && !msg->get_wind_dir_var())
+                    msg->set_wind_dir_var(*var);
 
-                        var = sfc->find(WR_VAR(0, 11, 2));
-                        if (var && !msg->get_wind_speed_var())
-                                msg->set_wind_speed_var(*var);
-                }
+                var = sfc->find(WR_VAR(0, 11, 2));
+                if (var && !msg->get_wind_speed_var())
+                    msg->set_wind_speed_var(*var);
+            }
         }
     }
 
     MsgType scanType(const Bulletin& bulletin) const
     {
-        switch (bulletin.type)
+        switch (bulletin.data_category)
         {
             case 2:
-                switch (bulletin.subtype)
+                switch (bulletin.data_subcategory)
                 {
                     case 1: // 001 for PILOT data,
                     case 2: // 002 for PILOT SHIP data,
@@ -206,7 +207,7 @@ public:
                     case 4: return MSG_TEMP;
                     case 5: return MSG_TEMP_SHIP;
                     case 255:
-                        switch (bulletin.localsubtype)
+                        switch (bulletin.data_subcategory_local)
                         {
                             case 0: {
                                 /* Guess looking at the variables */
@@ -232,9 +233,9 @@ public:
     }
 };
 
-std::auto_ptr<Importer> Importer::createTemp(const msg::Importer::Options& opts)
+std::unique_ptr<Importer> Importer::createTemp(const msg::Importer::Options& opts)
 {
-    return auto_ptr<Importer>(new TempImporter(opts));
+    return unique_ptr<Importer>(new TempImporter(opts));
 }
 
 void TempImporter::import_var(const Var& var)
@@ -250,7 +251,7 @@ void TempImporter::import_var(const Var& var)
 /* Date/time of launch */
         case WR_VAR(0,  8, 21):
             if (var.enqi() != 18)
-                    error_consistency::throwf("TEMP time significance is %d instead of 18", var.enqi());
+                dba_verbose(DBA_VERB_BUFREX_MSG, "TEMP time significance is %d instead of 18", var.enqi());
             break;
 /* Horizontal and vertical coordinates of launch site */
         case WR_VAR(0,  7,  1):
@@ -307,9 +308,9 @@ void TempImporter::import_var(const Var& var)
                 unsigned val = convert_BUFR08001_to_BUFR08042(var.enqi());
                 if (val != BUFR08042::ALL_MISSING)
                 {
-                    auto_ptr<Var> nvar(newvar(WR_VAR(0, 8, 42), (int)val));
-                    nvar->copy_attrs(var);
-                    msg->set(nvar, Level(100, press), Trange::instant());
+                    unique_ptr<Var> nvar(newvar(WR_VAR(0, 8, 42), (int)val));
+                    nvar->setattrs(var);
+                    msg->set(move(nvar), Level(100, press), Trange::instant());
                 }
             }
             if (var.enqi() & BUFR08001::SURFACE)
@@ -418,9 +419,9 @@ void TempImporter::import_group(unsigned start, unsigned length)
                         msg->seti(WR_VAR(0,  8,  42), BUFR08042::MISSING, -1, lev, Trange::instant());
                     else
                     {
-                        auto_ptr<Var> nvar(newvar(WR_VAR(0, 8, 42), (int)val));
-                        nvar->copy_attrs(var);
-                        msg->set(nvar, lev, Trange::instant());
+                        unique_ptr<Var> nvar(newvar(WR_VAR(0, 8, 42), (int)val));
+                        nvar->setattrs(var);
+                        msg->set(move(nvar), lev, Trange::instant());
                     }
                 }
                 break;
