@@ -94,7 +94,7 @@
 !     Insert some QC flags
       call idba_setc(handle, "*B33002", "1")
       call ensure_no_error("critica setc 0")
-      call idba_setc(handle, "*B33003", "t")
+      call idba_setc(handle, "*B33003", "1")
       call ensure_no_error("critica setc 1")
       call idba_setc(handle, "*var", "B01011")
       call idba_critica(handle)
